# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: yocto_i2cport_aio.py 66072 2025-04-30 06:59:12Z mvuilleu $
#
#  Implements the asyncio YI2cPort API for I2cPort functions
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: Asyncio implementation of YI2cPort
version: PATCH_WITH_VERSION
requires: yocto_api_aio
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api_aio import const, _IS_MICROPYTHON
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa

from .yocto_api_aio import (
    YAPIContext, YAPI, YAPI_Exception, YFunction, HwId, hwid2str,
    xarray, xbytearray, XStringIO
)

# --- (generated code: YI2cSnoopingRecord class start)
# noinspection PyProtectedMember
class YI2cSnoopingRecord:
    # --- (end of generated code: YI2cSnoopingRecord class start)
    if not _IS_MICROPYTHON:
        # --- (generated code: YI2cSnoopingRecord return codes)
        pass
        # --- (end of generated code: YI2cSnoopingRecord return codes)
        pass

    # --- (generated code: YI2cSnoopingRecord attributes declaration)
    _tim: int
    _pos: int
    _dir: int
    _msg: str
    # --- (end of generated code: YI2cSnoopingRecord attributes declaration)


    def __init__(self, json_data: xarray):
        # --- (generated code: YI2cSnoopingRecord constructor)
        self._tim = 0
        self._pos = 0
        self._dir = 0
        self._msg = ''
        # --- (end of generated code: YI2cSnoopingRecord constructor)
        json_val: Any = json.load(XStringIO(json_data))
        if 't' in json_val:
            self._tim =json_val["t"]
        if 'p' in json_val:
            self._pos = json_val["p"]
        if 'm' in json_val:
            m : str = json_val["m"]
            if m[0] == '<':
                self._dir = 1
            else:
                self._dir = 0
            self._msg = m[1:]

    # --- (generated code: YI2cSnoopingRecord implementation)
    def get_time(self) -> int:
        """
        Returns the elapsed time, in ms, since the beginning of the preceding message.

        @return the elapsed time, in ms, since the beginning of the preceding message.
        """
        return self._tim

    def get_pos(self) -> int:
        """
        Returns the absolute position of the message end.

        @return the absolute position of the message end.
        """
        return self._pos

    def get_direction(self) -> int:
        """
        Returns the message direction (RX=0, TX=1).

        @return the message direction (RX=0, TX=1).
        """
        return self._dir

    def get_message(self) -> str:
        """
        Returns the message content.

        @return the message content.
        """
        return self._msg

    # --- (end of generated code: YI2cSnoopingRecord implementation)


# --- (generated code: YI2cPort class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YI2cPortValueCallback = Union[Callable[['YI2cPort', str], Awaitable[None]], None]
    except TypeError:
        YI2cPortValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YI2cPort(YFunction):
    """
    The YI2cPort classe allows you to fully drive a Yoctopuce I2C port.
    It can be used to send and receive data, and to configure communication
    parameters (baud rate, etc).
    Note that Yoctopuce I2C ports are not exposed as virtual COM ports.
    They are meant to be used in the same way as all Yoctopuce devices.

    """
    # --- (end of generated code: YI2cPort class start)
    if not _IS_MICROPYTHON:
        # --- (generated code: YI2cPort return codes)
        RXCOUNT_INVALID: Final[int] = YAPI.INVALID_UINT
        TXCOUNT_INVALID: Final[int] = YAPI.INVALID_UINT
        ERRCOUNT_INVALID: Final[int] = YAPI.INVALID_UINT
        RXMSGCOUNT_INVALID: Final[int] = YAPI.INVALID_UINT
        TXMSGCOUNT_INVALID: Final[int] = YAPI.INVALID_UINT
        LASTMSG_INVALID: Final[str] = YAPI.INVALID_STRING
        CURRENTJOB_INVALID: Final[str] = YAPI.INVALID_STRING
        STARTUPJOB_INVALID: Final[str] = YAPI.INVALID_STRING
        JOBMAXTASK_INVALID: Final[int] = YAPI.INVALID_UINT
        JOBMAXSIZE_INVALID: Final[int] = YAPI.INVALID_UINT
        COMMAND_INVALID: Final[str] = YAPI.INVALID_STRING
        PROTOCOL_INVALID: Final[str] = YAPI.INVALID_STRING
        I2CMODE_INVALID: Final[str] = YAPI.INVALID_STRING
        I2CVOLTAGELEVEL_OFF: Final[int] = 0
        I2CVOLTAGELEVEL_3V3: Final[int] = 1
        I2CVOLTAGELEVEL_1V8: Final[int] = 2
        I2CVOLTAGELEVEL_INVALID: Final[int] = -1
        # --- (end of generated code: YI2cPort return codes)

    # --- (generated code: YI2cPort attributes declaration)
    _rxCount: int
    _txCount: int
    _errCount: int
    _rxMsgCount: int
    _txMsgCount: int
    _lastMsg: str
    _currentJob: str
    _startupJob: str
    _jobMaxTask: int
    _jobMaxSize: int
    _command: str
    _protocol: str
    _i2cVoltageLevel: int
    _i2cMode: str
    _valueCallback: YI2cPortValueCallback
    _rxptr: int
    _rxbuff: xarray
    _rxbuffptr: int
    # --- (end of generated code: YI2cPort attributes declaration)


    def __init__(self, yctx: YAPIContext, func: str):
        super().__init__(yctx, func)
        self._className = 'I2cPort'
        # --- (generated code: YI2cPort constructor)
        self._rxCount = YI2cPort.RXCOUNT_INVALID
        self._txCount = YI2cPort.TXCOUNT_INVALID
        self._errCount = YI2cPort.ERRCOUNT_INVALID
        self._rxMsgCount = YI2cPort.RXMSGCOUNT_INVALID
        self._txMsgCount = YI2cPort.TXMSGCOUNT_INVALID
        self._lastMsg = YI2cPort.LASTMSG_INVALID
        self._currentJob = YI2cPort.CURRENTJOB_INVALID
        self._startupJob = YI2cPort.STARTUPJOB_INVALID
        self._jobMaxTask = YI2cPort.JOBMAXTASK_INVALID
        self._jobMaxSize = YI2cPort.JOBMAXSIZE_INVALID
        self._command = YI2cPort.COMMAND_INVALID
        self._protocol = YI2cPort.PROTOCOL_INVALID
        self._i2cVoltageLevel = YI2cPort.I2CVOLTAGELEVEL_INVALID
        self._i2cMode = YI2cPort.I2CMODE_INVALID
        self._rxptr = 0
        self._rxbuff = xbytearray(0)
        self._rxbuffptr = 0
        # --- (end of generated code: YI2cPort constructor)

    # --- (generated code: YI2cPort implementation)

    @staticmethod
    def FirstI2cPort() -> Union[YI2cPort, None]:
        """
        Starts the enumeration of I2C ports currently accessible.
        Use the method YI2cPort.nextI2cPort() to iterate on
        next I2C ports.

        @return a pointer to a YI2cPort object, corresponding to
                the first I2C port currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = YAPI._yHash.getFirstHardwareId('I2cPort')
        if not next_hwid:
            return None
        return YI2cPort.FindI2cPort(hwid2str(next_hwid))

    @staticmethod
    def FirstI2cPortInContext(yctx: YAPIContext) -> Union[YI2cPort, None]:
        """
        Starts the enumeration of I2C ports currently accessible.
        Use the method YI2cPort.nextI2cPort() to iterate on
        next I2C ports.

        @param yctx : a YAPI context.

        @return a pointer to a YI2cPort object, corresponding to
                the first I2C port currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = yctx._yHash.getFirstHardwareId('I2cPort')
        if not next_hwid:
            return None
        return YI2cPort.FindI2cPortInContext(yctx, hwid2str(next_hwid))

    def nextI2cPort(self):
        """
        Continues the enumeration of I2C ports started using yFirstI2cPort().
        Caution: You can't make any assumption about the returned I2C ports order.
        If you want to find a specific an I2C port, use I2cPort.findI2cPort()
        and a hardwareID or a logical name.

        @return a pointer to a YI2cPort object, corresponding to
                an I2C port currently online, or a None pointer
                if there are no more I2C ports to enumerate.
        """
        next_hwid: Union[HwId, None] = None
        try:
            hwid: HwId = self._yapi._yHash.resolveHwID(self._className, self._func)
            next_hwid = self._yapi._yHash.getNextHardwareId(self._className, hwid)
        except YAPI_Exception:
            pass
        if not next_hwid:
            return None
        return YI2cPort.FindI2cPortInContext(self._yapi, hwid2str(next_hwid))

    def _parseAttr(self, json_val: dict) -> None:
        if 'rxCount' in json_val:
            self._rxCount = json_val["rxCount"]
        if 'txCount' in json_val:
            self._txCount = json_val["txCount"]
        if 'errCount' in json_val:
            self._errCount = json_val["errCount"]
        if 'rxMsgCount' in json_val:
            self._rxMsgCount = json_val["rxMsgCount"]
        if 'txMsgCount' in json_val:
            self._txMsgCount = json_val["txMsgCount"]
        if 'lastMsg' in json_val:
            self._lastMsg = json_val["lastMsg"]
        if 'currentJob' in json_val:
            self._currentJob = json_val["currentJob"]
        if 'startupJob' in json_val:
            self._startupJob = json_val["startupJob"]
        if 'jobMaxTask' in json_val:
            self._jobMaxTask = json_val["jobMaxTask"]
        if 'jobMaxSize' in json_val:
            self._jobMaxSize = json_val["jobMaxSize"]
        if 'command' in json_val:
            self._command = json_val["command"]
        if 'protocol' in json_val:
            self._protocol = json_val["protocol"]
        if 'i2cVoltageLevel' in json_val:
            self._i2cVoltageLevel = json_val["i2cVoltageLevel"]
        if 'i2cMode' in json_val:
            self._i2cMode = json_val["i2cMode"]
        super()._parseAttr(json_val)

    async def get_rxCount(self) -> int:
        """
        Returns the total number of bytes received since last reset.

        @return an integer corresponding to the total number of bytes received since last reset

        On failure, throws an exception or returns YI2cPort.RXCOUNT_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YI2cPort.RXCOUNT_INVALID
        res = self._rxCount
        return res

    async def get_txCount(self) -> int:
        """
        Returns the total number of bytes transmitted since last reset.

        @return an integer corresponding to the total number of bytes transmitted since last reset

        On failure, throws an exception or returns YI2cPort.TXCOUNT_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YI2cPort.TXCOUNT_INVALID
        res = self._txCount
        return res

    async def get_errCount(self) -> int:
        """
        Returns the total number of communication errors detected since last reset.

        @return an integer corresponding to the total number of communication errors detected since last reset

        On failure, throws an exception or returns YI2cPort.ERRCOUNT_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YI2cPort.ERRCOUNT_INVALID
        res = self._errCount
        return res

    async def get_rxMsgCount(self) -> int:
        """
        Returns the total number of messages received since last reset.

        @return an integer corresponding to the total number of messages received since last reset

        On failure, throws an exception or returns YI2cPort.RXMSGCOUNT_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YI2cPort.RXMSGCOUNT_INVALID
        res = self._rxMsgCount
        return res

    async def get_txMsgCount(self) -> int:
        """
        Returns the total number of messages send since last reset.

        @return an integer corresponding to the total number of messages send since last reset

        On failure, throws an exception or returns YI2cPort.TXMSGCOUNT_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YI2cPort.TXMSGCOUNT_INVALID
        res = self._txMsgCount
        return res

    async def get_lastMsg(self) -> str:
        """
        Returns the latest message fully received (for Line and Frame protocols).

        @return a string corresponding to the latest message fully received (for Line and Frame protocols)

        On failure, throws an exception or returns YI2cPort.LASTMSG_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YI2cPort.LASTMSG_INVALID
        res = self._lastMsg
        return res

    async def get_currentJob(self) -> str:
        """
        Returns the name of the job file currently in use.

        @return a string corresponding to the name of the job file currently in use

        On failure, throws an exception or returns YI2cPort.CURRENTJOB_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YI2cPort.CURRENTJOB_INVALID
        res = self._currentJob
        return res

    async def set_currentJob(self, newval: str) -> int:
        """
        Selects a job file to run immediately. If an empty string is
        given as argument, stops running current job file.

        @param newval : a string

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = newval
        return await self._setAttr("currentJob", rest_val)

    async def get_startupJob(self) -> str:
        """
        Returns the job file to use when the device is powered on.

        @return a string corresponding to the job file to use when the device is powered on

        On failure, throws an exception or returns YI2cPort.STARTUPJOB_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YI2cPort.STARTUPJOB_INVALID
        res = self._startupJob
        return res

    async def set_startupJob(self, newval: str) -> int:
        """
        Changes the job to use when the device is powered on.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : a string corresponding to the job to use when the device is powered on

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = newval
        return await self._setAttr("startupJob", rest_val)

    async def get_jobMaxTask(self) -> int:
        """
        Returns the maximum number of tasks in a job that the device can handle.

        @return an integer corresponding to the maximum number of tasks in a job that the device can handle

        On failure, throws an exception or returns YI2cPort.JOBMAXTASK_INVALID.
        """
        res: int
        if self._cacheExpiration == 0:
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YI2cPort.JOBMAXTASK_INVALID
        res = self._jobMaxTask
        return res

    async def get_jobMaxSize(self) -> int:
        """
        Returns maximum size allowed for job files.

        @return an integer corresponding to maximum size allowed for job files

        On failure, throws an exception or returns YI2cPort.JOBMAXSIZE_INVALID.
        """
        res: int
        if self._cacheExpiration == 0:
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YI2cPort.JOBMAXSIZE_INVALID
        res = self._jobMaxSize
        return res

    async def get_command(self) -> str:
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YI2cPort.COMMAND_INVALID
        res = self._command
        return res

    async def set_command(self, newval: str) -> int:
        rest_val = newval
        return await self._setAttr("command", rest_val)

    async def get_protocol(self) -> str:
        """
        Returns the type of protocol used to send I2C messages, as a string.
        Possible values are
        "Line" for messages separated by LF or
        "Char" for continuous stream of codes.

        @return a string corresponding to the type of protocol used to send I2C messages, as a string

        On failure, throws an exception or returns YI2cPort.PROTOCOL_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YI2cPort.PROTOCOL_INVALID
        res = self._protocol
        return res

    async def set_protocol(self, newval: str) -> int:
        """
        Changes the type of protocol used to send I2C messages.
        Possible values are
        "Line" for messages separated by LF or
        "Char" for continuous stream of codes.
        The suffix "/[wait]ms" can be added to reduce the transmit rate so that there
        is always at lest the specified number of milliseconds between each message sent.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : a string corresponding to the type of protocol used to send I2C messages

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = newval
        return await self._setAttr("protocol", rest_val)

    async def get_i2cVoltageLevel(self) -> int:
        """
        Returns the voltage level used on the I2C bus.

        @return a value among YI2cPort.I2CVOLTAGELEVEL_OFF, YI2cPort.I2CVOLTAGELEVEL_3V3 and
        YI2cPort.I2CVOLTAGELEVEL_1V8 corresponding to the voltage level used on the I2C bus

        On failure, throws an exception or returns YI2cPort.I2CVOLTAGELEVEL_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YI2cPort.I2CVOLTAGELEVEL_INVALID
        res = self._i2cVoltageLevel
        return res

    async def set_i2cVoltageLevel(self, newval: int) -> int:
        """
        Changes the voltage level used on the I2C bus.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : a value among YI2cPort.I2CVOLTAGELEVEL_OFF, YI2cPort.I2CVOLTAGELEVEL_3V3 and
        YI2cPort.I2CVOLTAGELEVEL_1V8 corresponding to the voltage level used on the I2C bus

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("i2cVoltageLevel", rest_val)

    async def get_i2cMode(self) -> str:
        """
        Returns the I2C port communication parameters, as a string such as
        "400kbps,2000ms,NoRestart". The string includes the baud rate, the
        recovery delay after communications errors, and if needed the option
        NoRestart to use a Stop/Start sequence instead of the
        Restart state when performing read on the I2C bus.

        @return a string corresponding to the I2C port communication parameters, as a string such as
                "400kbps,2000ms,NoRestart"

        On failure, throws an exception or returns YI2cPort.I2CMODE_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YI2cPort.I2CMODE_INVALID
        res = self._i2cMode
        return res

    async def set_i2cMode(self, newval: str) -> int:
        """
        Changes the I2C port communication parameters, with a string such as
        "400kbps,2000ms". The string includes the baud rate, the
        recovery delay after communications errors, and if needed the option
        NoRestart to use a Stop/Start sequence instead of the
        Restart state when performing read on the I2C bus.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : a string corresponding to the I2C port communication parameters, with a string such as
                "400kbps,2000ms"

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = newval
        return await self._setAttr("i2cMode", rest_val)

    @staticmethod
    def FindI2cPort(func: str) -> YI2cPort:
        """
        Retrieves an I2C port for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the I2C port is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YI2cPort.isOnline() to test if the I2C port is
        indeed online at a given time. In case of ambiguity when looking for
        an I2C port by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the I2C port, for instance
                YI2CMK01.i2cPort.

        @return a YI2cPort object allowing you to drive the I2C port.
        """
        obj: Union[YI2cPort, None]
        obj = YFunction._FindFromCache("I2cPort", func)
        if obj is None:
            obj = YI2cPort(YAPI, func)
            YFunction._AddToCache("I2cPort", func, obj)
        return obj

    @staticmethod
    def FindI2cPortInContext(yctx: YAPIContext, func: str) -> YI2cPort:
        """
        Retrieves an I2C port for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the I2C port is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YI2cPort.isOnline() to test if the I2C port is
        indeed online at a given time. In case of ambiguity when looking for
        an I2C port by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the I2C port, for instance
                YI2CMK01.i2cPort.

        @return a YI2cPort object allowing you to drive the I2C port.
        """
        obj: Union[YI2cPort, None]
        obj = YFunction._FindFromCacheInContext(yctx, "I2cPort", func)
        if obj is None:
            obj = YI2cPort(yctx, func)
            YFunction._AddToCache("I2cPort", func, obj)
        return obj

    if not _IS_MICROPYTHON:
        async def registerValueCallback(self, callback: YI2cPortValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerValueCallback(callback)

    async def sendCommand(self, text: str) -> int:
        return await self.set_command(text)

    async def readLine(self) -> str:
        """
        Reads a single line (or message) from the receive buffer, starting at current stream position.
        This function is intended to be used when the serial port is configured for a message protocol,
        such as 'Line' mode or frame protocols.

        If data at current stream position is not available anymore in the receive buffer,
        the function returns the oldest available line and moves the stream position just after.
        If no new full line is received, the function returns an empty line.

        @return a string with a single line of text

        On failure, throws an exception or returns a negative error code.
        """
        url: str
        msgbin: xarray
        msgarr: list[xarray] = []
        msglen: int
        res: str

        url = "rxmsg.json?pos=%d&len=1&maxw=1" % self._rxptr
        msgbin = await self._download(url)
        msgarr = self._json_get_array(msgbin)
        msglen = len(msgarr)
        if msglen == 0:
            return ""
        # last element of array is the new position
        msglen = msglen - 1
        self._rxptr = self._decode_json_int(msgarr[msglen])
        if msglen == 0:
            return ""
        res = self._json_get_string(msgarr[0])
        return res

    async def readMessages(self, pattern: str, maxWait: int) -> list[str]:
        """
        Searches for incoming messages in the serial port receive buffer matching a given pattern,
        starting at current position. This function will only compare and return printable characters
        in the message strings. Binary protocols are handled as hexadecimal strings.

        The search returns all messages matching the expression provided as argument in the buffer.
        If no matching message is found, the search waits for one up to the specified maximum timeout
        (in milliseconds).

        @param pattern : a limited regular expression describing the expected message format,
                or an empty string if all messages should be returned (no filtering).
                When using binary protocols, the format applies to the hexadecimal
                representation of the message.
        @param maxWait : the maximum number of milliseconds to wait for a message if none is found
                in the receive buffer.

        @return an array of strings containing the messages found, if any.
                Binary messages are converted to hexadecimal representation.

        On failure, throws an exception or returns an empty array.
        """
        url: str
        msgbin: xarray
        msgarr: list[xarray] = []
        msglen: int
        res: list[str] = []
        idx: int

        url = "rxmsg.json?pos=%d&maxw=%d&pat=%s" % (self._rxptr, maxWait, pattern)
        msgbin = await self._download(url)
        msgarr = self._json_get_array(msgbin)
        msglen = len(msgarr)
        if msglen == 0:
            return res
        # last element of array is the new position
        msglen = msglen - 1
        self._rxptr = self._decode_json_int(msgarr[msglen])
        idx = 0

        while idx < msglen:
            res.append(self._json_get_string(msgarr[idx]))
            idx = idx + 1

        return res

    async def read_seek(self, absPos: int) -> int:
        """
        Changes the current internal stream position to the specified value. This function
        does not affect the device, it only changes the value stored in the API object
        for the next read operations.

        @param absPos : the absolute position index for next read operations.

        @return nothing.
        """
        self._rxptr = absPos
        return YAPI.SUCCESS

    async def read_tell(self) -> int:
        """
        Returns the current absolute stream position pointer of the API object.

        @return the absolute position index for next read operations.
        """
        return self._rxptr

    async def read_avail(self) -> int:
        """
        Returns the number of bytes available to read in the input buffer starting from the
        current absolute stream position pointer of the API object.

        @return the number of bytes available to read
        """
        availPosStr: str
        atPos: int
        res: int
        databin: xarray

        databin = await self._download("rxcnt.bin?pos=%d" % self._rxptr)
        availPosStr = databin.decode('latin-1')
        atPos = availPosStr.find("@")
        res = YAPI._atoi(availPosStr[0: 0 + atPos])
        return res

    async def end_tell(self) -> int:
        availPosStr: str
        atPos: int
        res: int
        databin: xarray

        databin = await self._download("rxcnt.bin?pos=%d" % self._rxptr)
        availPosStr = databin.decode('latin-1')
        atPos = availPosStr.find("@")
        res = YAPI._atoi(availPosStr[atPos+1: atPos+1 + len(availPosStr)-atPos-1])
        return res

    async def queryLine(self, query: str, maxWait: int) -> str:
        """
        Sends a text line query to the serial port, and reads the reply, if any.
        This function is intended to be used when the serial port is configured for 'Line' protocol.

        @param query : the line query to send (without CR/LF)
        @param maxWait : the maximum number of milliseconds to wait for a reply.

        @return the next text line received after sending the text query, as a string.
                Additional lines can be obtained by calling readLine or readMessages.

        On failure, throws an exception or returns an empty string.
        """
        prevpos: int
        url: str
        msgbin: xarray
        msgarr: list[xarray] = []
        msglen: int
        res: str
        if len(query) <= 80:
            # fast query
            url = "rxmsg.json?len=1&maxw=%d&cmd=!%s" % (maxWait, self._escapeAttr(query))
        else:
            # long query
            prevpos = await self.end_tell()
            await self._upload("txdata", xbytearray(query + "\r\n", 'latin-1'))
            url = "rxmsg.json?len=1&maxw=%d&pos=%d" % (maxWait, prevpos)

        msgbin = await self._download(url)
        msgarr = self._json_get_array(msgbin)
        msglen = len(msgarr)
        if msglen == 0:
            return ""
        # last element of array is the new position
        msglen = msglen - 1
        self._rxptr = self._decode_json_int(msgarr[msglen])
        if msglen == 0:
            return ""
        res = self._json_get_string(msgarr[0])
        return res

    async def queryHex(self, hexString: str, maxWait: int) -> str:
        """
        Sends a binary message to the serial port, and reads the reply, if any.
        This function is intended to be used when the serial port is configured for
        Frame-based protocol.

        @param hexString : the message to send, coded in hexadecimal
        @param maxWait : the maximum number of milliseconds to wait for a reply.

        @return the next frame received after sending the message, as a hex string.
                Additional frames can be obtained by calling readHex or readMessages.

        On failure, throws an exception or returns an empty string.
        """
        prevpos: int
        url: str
        msgbin: xarray
        msgarr: list[xarray] = []
        msglen: int
        res: str
        if len(hexString) <= 80:
            # fast query
            url = "rxmsg.json?len=1&maxw=%d&cmd=$%s" % (maxWait, hexString)
        else:
            # long query
            prevpos = await self.end_tell()
            await self._upload("txdata", YAPI._hexStrToBin(hexString))
            url = "rxmsg.json?len=1&maxw=%d&pos=%d" % (maxWait, prevpos)

        msgbin = await self._download(url)
        msgarr = self._json_get_array(msgbin)
        msglen = len(msgarr)
        if msglen == 0:
            return ""
        # last element of array is the new position
        msglen = msglen - 1
        self._rxptr = self._decode_json_int(msgarr[msglen])
        if msglen == 0:
            return ""
        res = self._json_get_string(msgarr[0])
        return res

    async def uploadJob(self, jobfile: str, jsonDef: str) -> int:
        """
        Saves the job definition string (JSON data) into a job file.
        The job file can be later enabled using selectJob().

        @param jobfile : name of the job file to save on the device filesystem
        @param jsonDef : a string containing a JSON definition of the job

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        await self._upload(jobfile, xbytearray(jsonDef, 'latin-1'))
        return YAPI.SUCCESS

    async def selectJob(self, jobfile: str) -> int:
        """
        Load and start processing the specified job file. The file must have
        been previously created using the user interface or uploaded on the
        device filesystem using the uploadJob() function.

        @param jobfile : name of the job file (on the device filesystem)

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.set_currentJob(jobfile)

    async def reset(self) -> int:
        """
        Clears the serial port buffer and resets counters to zero.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        self._rxptr = 0
        self._rxbuffptr = 0
        self._rxbuff = xbytearray(0)

        return await self.sendCommand("Z")

    async def i2cSendBin(self, slaveAddr: int, buff: xarray) -> int:
        """
        Sends a one-way message (provided as a a binary buffer) to a device on the I2C bus.
        This function checks and reports communication errors on the I2C bus.

        @param slaveAddr : the 7-bit address of the slave device (without the direction bit)
        @param buff : the binary buffer to be sent

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        nBytes: int
        idx: int
        val: int
        msg: str
        reply: str
        msg = "@%02x:" % slaveAddr
        nBytes = len(buff)
        idx = 0
        while idx < nBytes:
            val = buff[idx]
            msg = "%s%02x" % (msg, val)
            idx = idx + 1

        reply = await self.queryLine(msg, 1000)
        if not (len(reply) > 0):
            self._throw(YAPI.IO_ERROR, "No response from I2C device")
            return YAPI.IO_ERROR
        idx = reply.find("[N]!")
        if not (idx < 0):
            self._throw(YAPI.IO_ERROR, "No I2C ACK received")
            return YAPI.IO_ERROR
        idx = reply.find("!")
        if not (idx < 0):
            self._throw(YAPI.IO_ERROR, "I2C protocol error")
            return YAPI.IO_ERROR
        return YAPI.SUCCESS

    async def i2cSendArray(self, slaveAddr: int, values: list[int]) -> int:
        """
        Sends a one-way message (provided as a list of integer) to a device on the I2C bus.
        This function checks and reports communication errors on the I2C bus.

        @param slaveAddr : the 7-bit address of the slave device (without the direction bit)
        @param values : a list of data bytes to be sent

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        nBytes: int
        idx: int
        val: int
        msg: str
        reply: str
        msg = "@%02x:" % slaveAddr
        nBytes = len(values)
        idx = 0
        while idx < nBytes:
            val = values[idx]
            msg = "%s%02x" % (msg, val)
            idx = idx + 1

        reply = await self.queryLine(msg, 1000)
        if not (len(reply) > 0):
            self._throw(YAPI.IO_ERROR, "No response from I2C device")
            return YAPI.IO_ERROR
        idx = reply.find("[N]!")
        if not (idx < 0):
            self._throw(YAPI.IO_ERROR, "No I2C ACK received")
            return YAPI.IO_ERROR
        idx = reply.find("!")
        if not (idx < 0):
            self._throw(YAPI.IO_ERROR, "I2C protocol error")
            return YAPI.IO_ERROR
        return YAPI.SUCCESS

    async def i2cSendAndReceiveBin(self, slaveAddr: int, buff: xarray, rcvCount: int) -> xarray:
        """
        Sends a one-way message (provided as a a binary buffer) to a device on the I2C bus,
        then read back the specified number of bytes from device.
        This function checks and reports communication errors on the I2C bus.

        @param slaveAddr : the 7-bit address of the slave device (without the direction bit)
        @param buff : the binary buffer to be sent
        @param rcvCount : the number of bytes to receive once the data bytes are sent

        @return a list of bytes with the data received from slave device.

        On failure, throws an exception or returns an empty binary buffer.
        """
        nBytes: int
        idx: int
        val: int
        msg: str
        reply: str
        rcvbytes: xarray
        rcvbytes = xbytearray(0)
        if not (rcvCount<=512):
            self._throw(YAPI.INVALID_ARGUMENT, "Cannot read more than 512 bytes")
            return rcvbytes
        msg = "@%02x:" % slaveAddr
        nBytes = len(buff)
        idx = 0
        while idx < nBytes:
            val = buff[idx]
            msg = "%s%02x" % (msg, val)
            idx = idx + 1
        idx = 0
        if rcvCount > 54:
            while rcvCount - idx > 255:
                msg = "%sxx*FF" % msg
                idx = idx + 255
            if rcvCount - idx > 2:
                msg = "%sxx*%02X" % (msg, (rcvCount - idx))
                idx = rcvCount
        while idx < rcvCount:
            msg = "%sxx" % msg
            idx = idx + 1

        reply = await self.queryLine(msg, 1000)
        if not (len(reply) > 0):
            self._throw(YAPI.IO_ERROR, "No response from I2C device")
            return rcvbytes
        idx = reply.find("[N]!")
        if not (idx < 0):
            self._throw(YAPI.IO_ERROR, "No I2C ACK received")
            return rcvbytes
        idx = reply.find("!")
        if not (idx < 0):
            self._throw(YAPI.IO_ERROR, "I2C protocol error")
            return rcvbytes
        reply = reply[len(reply)-2*rcvCount: len(reply)-2*rcvCount + 2*rcvCount]
        rcvbytes = YAPI._hexStrToBin(reply)
        return rcvbytes

    async def i2cSendAndReceiveArray(self, slaveAddr: int, values: list[int], rcvCount: int) -> list[int]:
        """
        Sends a one-way message (provided as a list of integer) to a device on the I2C bus,
        then read back the specified number of bytes from device.
        This function checks and reports communication errors on the I2C bus.

        @param slaveAddr : the 7-bit address of the slave device (without the direction bit)
        @param values : a list of data bytes to be sent
        @param rcvCount : the number of bytes to receive once the data bytes are sent

        @return a list of bytes with the data received from slave device.

        On failure, throws an exception or returns an empty array.
        """
        nBytes: int
        idx: int
        val: int
        msg: str
        reply: str
        rcvbytes: xarray
        res: list[int] = []
        del res[:]
        if not (rcvCount<=512):
            self._throw(YAPI.INVALID_ARGUMENT, "Cannot read more than 512 bytes")
            return res
        msg = "@%02x:" % slaveAddr
        nBytes = len(values)
        idx = 0
        while idx < nBytes:
            val = values[idx]
            msg = "%s%02x" % (msg, val)
            idx = idx + 1
        idx = 0
        if rcvCount > 54:
            while rcvCount - idx > 255:
                msg = "%sxx*FF" % msg
                idx = idx + 255
            if rcvCount - idx > 2:
                msg = "%sxx*%02X" % (msg, (rcvCount - idx))
                idx = rcvCount
        while idx < rcvCount:
            msg = "%sxx" % msg
            idx = idx + 1

        reply = await self.queryLine(msg, 1000)
        if not (len(reply) > 0):
            self._throw(YAPI.IO_ERROR, "No response from I2C device")
            return res
        idx = reply.find("[N]!")
        if not (idx < 0):
            self._throw(YAPI.IO_ERROR, "No I2C ACK received")
            return res
        idx = reply.find("!")
        if not (idx < 0):
            self._throw(YAPI.IO_ERROR, "I2C protocol error")
            return res
        reply = reply[len(reply)-2*rcvCount: len(reply)-2*rcvCount + 2*rcvCount]
        rcvbytes = YAPI._hexStrToBin(reply)
        del res[:]
        idx = 0
        while idx < rcvCount:
            val = rcvbytes[idx]
            res.append(val)
            idx = idx + 1

        return res

    async def writeStr(self, codes: str) -> int:
        """
        Sends a text-encoded I2C code stream to the I2C bus, as is.
        An I2C code stream is a string made of hexadecimal data bytes,
        but that may also include the I2C state transitions code:
        "{S}" to emit a start condition,
        "{R}" for a repeated start condition,
        "{P}" for a stop condition,
        "xx" for receiving a data byte,
        "{A}" to ack a data byte received and
        "{N}" to nack a data byte received.
        If a newline ("\n") is included in the stream, the message
        will be terminated and a newline will also be added to the
        receive stream.

        @param codes : the code stream to send

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        bufflen: int
        buff: xarray
        idx: int
        ch: int
        buff = xbytearray(codes, 'latin-1')
        bufflen = len(buff)
        if bufflen < 100:
            # if string is pure text, we can send it as a simple command (faster)
            ch = 0x20
            idx = 0
            while (idx < bufflen) and(ch != 0):
                ch = buff[idx]
                if (ch >= 0x20) and(ch < 0x7f):
                    idx = idx + 1
                else:
                    ch = 0
            if idx >= bufflen:
                return await self.sendCommand("+%s" % codes)
        # send string using file upload
        return await self._upload("txdata", buff)

    async def writeLine(self, codes: str) -> int:
        """
        Sends a text-encoded I2C code stream to the I2C bus, and release the bus.
        An I2C code stream is a string made of hexadecimal data bytes,
        but that may also include the I2C state transitions code:
        "{S}" to emit a start condition,
        "{R}" for a repeated start condition,
        "{P}" for a stop condition,
        "xx" for receiving a data byte,
        "{A}" to ack a data byte received and
        "{N}" to nack a data byte received.
        At the end of the stream, a stop condition is added if missing
        and a newline is added to the receive buffer as well.

        @param codes : the code stream to send

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        bufflen: int
        buff: xarray
        bufflen = len(codes)
        if bufflen < 100:
            return await self.sendCommand("!%s" % codes)
        # send string using file upload
        buff = xbytearray("%s\n" % codes, 'latin-1')
        return await self._upload("txdata", buff)

    async def writeByte(self, code: int) -> int:
        """
        Sends a single byte to the I2C bus. Depending on the I2C bus state, the byte
        will be interpreted as an address byte or a data byte.

        @param code : the byte to send

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.sendCommand("+%02X" % code)

    async def writeHex(self, hexString: str) -> int:
        """
        Sends a byte sequence (provided as a hexadecimal string) to the I2C bus.
        Depending on the I2C bus state, the first byte will be interpreted as an
        address byte or a data byte.

        @param hexString : a string of hexadecimal byte codes

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        bufflen: int
        buff: xarray
        bufflen = len(hexString)
        if bufflen < 100:
            return await self.sendCommand("+%s" % hexString)
        buff = xbytearray(hexString, 'latin-1')

        return await self._upload("txdata", buff)

    async def writeBin(self, buff: xarray) -> int:
        """
        Sends a binary buffer to the I2C bus, as is.
        Depending on the I2C bus state, the first byte will be interpreted
        as an address byte or a data byte.

        @param buff : the binary buffer to send

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        nBytes: int
        idx: int
        val: int
        msg: str
        msg = ""
        nBytes = len(buff)
        idx = 0
        while idx < nBytes:
            val = buff[idx]
            msg = "%s%02x" % (msg, val)
            idx = idx + 1

        return await self.writeHex(msg)

    async def writeArray(self, byteList: list[int]) -> int:
        """
        Sends a byte sequence (provided as a list of bytes) to the I2C bus.
        Depending on the I2C bus state, the first byte will be interpreted as an
        address byte or a data byte.

        @param byteList : a list of byte codes

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        nBytes: int
        idx: int
        val: int
        msg: str
        msg = ""
        nBytes = len(byteList)
        idx = 0
        while idx < nBytes:
            val = byteList[idx]
            msg = "%s%02x" % (msg, val)
            idx = idx + 1

        return await self.writeHex(msg)

    async def snoopMessagesEx(self, maxWait: int, maxMsg: int) -> list[YI2cSnoopingRecord]:
        """
        Retrieves messages (both direction) in the I2C port buffer, starting at current position.

        If no message is found, the search waits for one up to the specified maximum timeout
        (in milliseconds).

        @param maxWait : the maximum number of milliseconds to wait for a message if none is found
                in the receive buffer.
        @param maxMsg : the maximum number of messages to be returned by the function; up to 254.

        @return an array of YI2cSnoopingRecord objects containing the messages found, if any.

        On failure, throws an exception or returns an empty array.
        """
        url: str
        msgbin: xarray
        msgarr: list[xarray] = []
        msglen: int
        res: Union[list[YI2cSnoopingRecord], None] = []
        idx: int

        url = "rxmsg.json?pos=%d&maxw=%d&t=0&len=%d" % (self._rxptr, maxWait, maxMsg)
        msgbin = await self._download(url)
        msgarr = self._json_get_array(msgbin)
        msglen = len(msgarr)
        if msglen == 0:
            return res
        # last element of array is the new position
        msglen = msglen - 1
        self._rxptr = self._decode_json_int(msgarr[msglen])
        idx = 0

        while idx < msglen:
            res.append(YI2cSnoopingRecord(msgarr[idx].decode('latin-1')))
            idx = idx + 1

        return res

    async def snoopMessages(self, maxWait: int) -> list[YI2cSnoopingRecord]:
        """
        Retrieves messages (both direction) in the I2C port buffer, starting at current position.

        If no message is found, the search waits for one up to the specified maximum timeout
        (in milliseconds).

        @param maxWait : the maximum number of milliseconds to wait for a message if none is found
                in the receive buffer.

        @return an array of YI2cSnoopingRecord objects containing the messages found, if any.

        On failure, throws an exception or returns an empty array.
        """
        return await self.snoopMessagesEx(maxWait, 255)

    # --- (end of generated code: YI2cPort implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  Asyncio implementation of YInputChain
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: Asyncio implementation of YInputChain
version: PATCH_WITH_VERSION
requires: yocto_api_aio
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api_aio import const, _IS_MICROPYTHON
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa

from .yocto_api_aio import (
    YAPIContext, YAPI, YAPI_Exception, HwId, hwid2str, YFunction, xarray
)

async def yInternalEventCallback(obj: YInputChain, value: str):
    await obj._internalEventHandler(value)

# --- (YInputChain class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YInputChainValueCallback = Union[Callable[['YInputChain', str], Awaitable[None]], None]
        YStateChangeCallback = Union[Callable[['YInputChain',int , str, str, str], Awaitable[None]], None]
    except TypeError:
        YInputChainValueCallback = Union[Callable, Awaitable]
        YStateChangeCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YInputChain(YFunction):
    """
    The YInputChain class provides access to separate
    digital inputs connected in a chain.

    """
    # --- (end of YInputChain class start)
    if not _IS_MICROPYTHON:
        # --- (YInputChain return codes)
        EXPECTEDNODES_INVALID: Final[int] = YAPI.INVALID_UINT
        DETECTEDNODES_INVALID: Final[int] = YAPI.INVALID_UINT
        REFRESHRATE_INVALID: Final[int] = YAPI.INVALID_UINT
        BITCHAIN1_INVALID: Final[str] = YAPI.INVALID_STRING
        BITCHAIN2_INVALID: Final[str] = YAPI.INVALID_STRING
        BITCHAIN3_INVALID: Final[str] = YAPI.INVALID_STRING
        BITCHAIN4_INVALID: Final[str] = YAPI.INVALID_STRING
        BITCHAIN5_INVALID: Final[str] = YAPI.INVALID_STRING
        BITCHAIN6_INVALID: Final[str] = YAPI.INVALID_STRING
        BITCHAIN7_INVALID: Final[str] = YAPI.INVALID_STRING
        WATCHDOGPERIOD_INVALID: Final[int] = YAPI.INVALID_UINT
        CHAINDIAGS_INVALID: Final[int] = YAPI.INVALID_UINT
        LOOPBACKTEST_OFF: Final[int] = 0
        LOOPBACKTEST_ON: Final[int] = 1
        LOOPBACKTEST_INVALID: Final[int] = -1
        # --- (end of YInputChain return codes)

    # --- (YInputChain attributes declaration)
    _expectedNodes: int
    _detectedNodes: int
    _loopbackTest: int
    _refreshRate: int
    _bitChain1: str
    _bitChain2: str
    _bitChain3: str
    _bitChain4: str
    _bitChain5: str
    _bitChain6: str
    _bitChain7: str
    _watchdogPeriod: int
    _chainDiags: int
    _valueCallback: YInputChainValueCallback
    _stateChangeCallback: YStateChangeCallback
    _prevPos: int
    _eventPos: int
    _eventStamp: int
    _eventChains: list[str]
    # --- (end of YInputChain attributes declaration)


    def __init__(self, yctx: YAPIContext, func: str):
        super().__init__(yctx, func)
        self._className = 'InputChain'
        # --- (YInputChain constructor)
        self._expectedNodes = YInputChain.EXPECTEDNODES_INVALID
        self._detectedNodes = YInputChain.DETECTEDNODES_INVALID
        self._loopbackTest = YInputChain.LOOPBACKTEST_INVALID
        self._refreshRate = YInputChain.REFRESHRATE_INVALID
        self._bitChain1 = YInputChain.BITCHAIN1_INVALID
        self._bitChain2 = YInputChain.BITCHAIN2_INVALID
        self._bitChain3 = YInputChain.BITCHAIN3_INVALID
        self._bitChain4 = YInputChain.BITCHAIN4_INVALID
        self._bitChain5 = YInputChain.BITCHAIN5_INVALID
        self._bitChain6 = YInputChain.BITCHAIN6_INVALID
        self._bitChain7 = YInputChain.BITCHAIN7_INVALID
        self._watchdogPeriod = YInputChain.WATCHDOGPERIOD_INVALID
        self._chainDiags = YInputChain.CHAINDIAGS_INVALID
        self._prevPos = 0
        self._eventPos = 0
        self._eventStamp = 0
        self._eventChains = []
        # --- (end of YInputChain constructor)

    # --- (YInputChain implementation)

    @staticmethod
    def FirstInputChain() -> Union[YInputChain, None]:
        """
        Starts the enumeration of digital input chains currently accessible.
        Use the method YInputChain.nextInputChain() to iterate on
        next digital input chains.

        @return a pointer to a YInputChain object, corresponding to
                the first digital input chain currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = YAPI._yHash.getFirstHardwareId('InputChain')
        if not next_hwid:
            return None
        return YInputChain.FindInputChain(hwid2str(next_hwid))

    @staticmethod
    def FirstInputChainInContext(yctx: YAPIContext) -> Union[YInputChain, None]:
        """
        Starts the enumeration of digital input chains currently accessible.
        Use the method YInputChain.nextInputChain() to iterate on
        next digital input chains.

        @param yctx : a YAPI context.

        @return a pointer to a YInputChain object, corresponding to
                the first digital input chain currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = yctx._yHash.getFirstHardwareId('InputChain')
        if not next_hwid:
            return None
        return YInputChain.FindInputChainInContext(yctx, hwid2str(next_hwid))

    def nextInputChain(self):
        """
        Continues the enumeration of digital input chains started using yFirstInputChain().
        Caution: You can't make any assumption about the returned digital input chains order.
        If you want to find a specific a digital input chain, use InputChain.findInputChain()
        and a hardwareID or a logical name.

        @return a pointer to a YInputChain object, corresponding to
                a digital input chain currently online, or a None pointer
                if there are no more digital input chains to enumerate.
        """
        next_hwid: Union[HwId, None] = None
        try:
            hwid: HwId = self._yapi._yHash.resolveHwID(self._className, self._func)
            next_hwid = self._yapi._yHash.getNextHardwareId(self._className, hwid)
        except YAPI_Exception:
            pass
        if not next_hwid:
            return None
        return YInputChain.FindInputChainInContext(self._yapi, hwid2str(next_hwid))

    def _parseAttr(self, json_val: dict) -> None:
        if 'expectedNodes' in json_val:
            self._expectedNodes = json_val["expectedNodes"]
        if 'detectedNodes' in json_val:
            self._detectedNodes = json_val["detectedNodes"]
        if 'loopbackTest' in json_val:
            self._loopbackTest = json_val["loopbackTest"] > 0
        if 'refreshRate' in json_val:
            self._refreshRate = json_val["refreshRate"]
        if 'bitChain1' in json_val:
            self._bitChain1 = json_val["bitChain1"]
        if 'bitChain2' in json_val:
            self._bitChain2 = json_val["bitChain2"]
        if 'bitChain3' in json_val:
            self._bitChain3 = json_val["bitChain3"]
        if 'bitChain4' in json_val:
            self._bitChain4 = json_val["bitChain4"]
        if 'bitChain5' in json_val:
            self._bitChain5 = json_val["bitChain5"]
        if 'bitChain6' in json_val:
            self._bitChain6 = json_val["bitChain6"]
        if 'bitChain7' in json_val:
            self._bitChain7 = json_val["bitChain7"]
        if 'watchdogPeriod' in json_val:
            self._watchdogPeriod = json_val["watchdogPeriod"]
        if 'chainDiags' in json_val:
            self._chainDiags = json_val["chainDiags"]
        super()._parseAttr(json_val)

    async def get_expectedNodes(self) -> int:
        """
        Returns the number of nodes expected in the chain.

        @return an integer corresponding to the number of nodes expected in the chain

        On failure, throws an exception or returns YInputChain.EXPECTEDNODES_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputChain.EXPECTEDNODES_INVALID
        res = self._expectedNodes
        return res

    async def set_expectedNodes(self, newval: int) -> int:
        """
        Changes the number of nodes expected in the chain.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : an integer corresponding to the number of nodes expected in the chain

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("expectedNodes", rest_val)

    async def get_detectedNodes(self) -> int:
        """
        Returns the number of nodes detected in the chain.

        @return an integer corresponding to the number of nodes detected in the chain

        On failure, throws an exception or returns YInputChain.DETECTEDNODES_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputChain.DETECTEDNODES_INVALID
        res = self._detectedNodes
        return res

    async def get_loopbackTest(self) -> int:
        """
        Returns the activation state of the exhaustive chain connectivity test.
        The connectivity test requires a cable connecting the end of the chain
        to the loopback test connector.

        @return either YInputChain.LOOPBACKTEST_OFF or YInputChain.LOOPBACKTEST_ON, according to the
        activation state of the exhaustive chain connectivity test

        On failure, throws an exception or returns YInputChain.LOOPBACKTEST_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputChain.LOOPBACKTEST_INVALID
        res = self._loopbackTest
        return res

    async def set_loopbackTest(self, newval: int) -> int:
        """
        Changes the activation state of the exhaustive chain connectivity test.
        The connectivity test requires a cable connecting the end of the chain
        to the loopback test connector.

        If you want the change to be kept after a device reboot,
        make sure  to call the matching module saveToFlash().

        @param newval : either YInputChain.LOOPBACKTEST_OFF or YInputChain.LOOPBACKTEST_ON, according to
        the activation state of the exhaustive chain connectivity test

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = "1" if newval > 0 else "0"
        return await self._setAttr("loopbackTest", rest_val)

    async def get_refreshRate(self) -> int:
        """
        Returns the desired refresh rate, measured in Hz.
        The higher the refresh rate is set, the higher the
        communication speed on the chain will be.

        @return an integer corresponding to the desired refresh rate, measured in Hz

        On failure, throws an exception or returns YInputChain.REFRESHRATE_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputChain.REFRESHRATE_INVALID
        res = self._refreshRate
        return res

    async def set_refreshRate(self, newval: int) -> int:
        """
        Changes the desired refresh rate, measured in Hz.
        The higher the refresh rate is set, the higher the
        communication speed on the chain will be.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : an integer corresponding to the desired refresh rate, measured in Hz

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("refreshRate", rest_val)

    async def get_bitChain1(self) -> str:
        """
        Returns the state of input 1 for all nodes of the input chain,
        as a hexadecimal string. The node nearest to the controller
        is the lowest bit of the result.

        @return a string corresponding to the state of input 1 for all nodes of the input chain,
                as a hexadecimal string

        On failure, throws an exception or returns YInputChain.BITCHAIN1_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputChain.BITCHAIN1_INVALID
        res = self._bitChain1
        return res

    async def get_bitChain2(self) -> str:
        """
        Returns the state of input 2 for all nodes of the input chain,
        as a hexadecimal string. The node nearest to the controller
        is the lowest bit of the result.

        @return a string corresponding to the state of input 2 for all nodes of the input chain,
                as a hexadecimal string

        On failure, throws an exception or returns YInputChain.BITCHAIN2_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputChain.BITCHAIN2_INVALID
        res = self._bitChain2
        return res

    async def get_bitChain3(self) -> str:
        """
        Returns the state of input 3 for all nodes of the input chain,
        as a hexadecimal string. The node nearest to the controller
        is the lowest bit of the result.

        @return a string corresponding to the state of input 3 for all nodes of the input chain,
                as a hexadecimal string

        On failure, throws an exception or returns YInputChain.BITCHAIN3_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputChain.BITCHAIN3_INVALID
        res = self._bitChain3
        return res

    async def get_bitChain4(self) -> str:
        """
        Returns the state of input 4 for all nodes of the input chain,
        as a hexadecimal string. The node nearest to the controller
        is the lowest bit of the result.

        @return a string corresponding to the state of input 4 for all nodes of the input chain,
                as a hexadecimal string

        On failure, throws an exception or returns YInputChain.BITCHAIN4_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputChain.BITCHAIN4_INVALID
        res = self._bitChain4
        return res

    async def get_bitChain5(self) -> str:
        """
        Returns the state of input 5 for all nodes of the input chain,
        as a hexadecimal string. The node nearest to the controller
        is the lowest bit of the result.

        @return a string corresponding to the state of input 5 for all nodes of the input chain,
                as a hexadecimal string

        On failure, throws an exception or returns YInputChain.BITCHAIN5_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputChain.BITCHAIN5_INVALID
        res = self._bitChain5
        return res

    async def get_bitChain6(self) -> str:
        """
        Returns the state of input 6 for all nodes of the input chain,
        as a hexadecimal string. The node nearest to the controller
        is the lowest bit of the result.

        @return a string corresponding to the state of input 6 for all nodes of the input chain,
                as a hexadecimal string

        On failure, throws an exception or returns YInputChain.BITCHAIN6_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputChain.BITCHAIN6_INVALID
        res = self._bitChain6
        return res

    async def get_bitChain7(self) -> str:
        """
        Returns the state of input 7 for all nodes of the input chain,
        as a hexadecimal string. The node nearest to the controller
        is the lowest bit of the result.

        @return a string corresponding to the state of input 7 for all nodes of the input chain,
                as a hexadecimal string

        On failure, throws an exception or returns YInputChain.BITCHAIN7_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputChain.BITCHAIN7_INVALID
        res = self._bitChain7
        return res

    async def get_watchdogPeriod(self) -> int:
        """
        Returns the wait time in seconds before triggering an inactivity
        timeout error.

        @return an integer corresponding to the wait time in seconds before triggering an inactivity
                timeout error

        On failure, throws an exception or returns YInputChain.WATCHDOGPERIOD_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputChain.WATCHDOGPERIOD_INVALID
        res = self._watchdogPeriod
        return res

    async def set_watchdogPeriod(self, newval: int) -> int:
        """
        Changes the wait time in seconds before triggering an inactivity
        timeout error. Remember to call the saveToFlash() method
        of the module if the modification must be kept.

        @param newval : an integer corresponding to the wait time in seconds before triggering an inactivity
                timeout error

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("watchdogPeriod", rest_val)

    async def get_chainDiags(self) -> int:
        """
        Returns the controller state diagnostics. Bit 0 indicates a chain length
        error, bit 1 indicates an inactivity timeout and bit 2 indicates
        a loopback test failure.

        @return an integer corresponding to the controller state diagnostics

        On failure, throws an exception or returns YInputChain.CHAINDIAGS_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputChain.CHAINDIAGS_INVALID
        res = self._chainDiags
        return res

    @staticmethod
    def FindInputChain(func: str) -> YInputChain:
        """
        Retrieves a digital input chain for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the digital input chain is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YInputChain.isOnline() to test if the digital input chain is
        indeed online at a given time. In case of ambiguity when looking for
        a digital input chain by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the digital input chain, for instance
                MyDevice.inputChain.

        @return a YInputChain object allowing you to drive the digital input chain.
        """
        obj: Union[YInputChain, None]
        obj = YFunction._FindFromCache("InputChain", func)
        if obj is None:
            obj = YInputChain(YAPI, func)
            YFunction._AddToCache("InputChain", func, obj)
        return obj

    @staticmethod
    def FindInputChainInContext(yctx: YAPIContext, func: str) -> YInputChain:
        """
        Retrieves a digital input chain for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the digital input chain is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YInputChain.isOnline() to test if the digital input chain is
        indeed online at a given time. In case of ambiguity when looking for
        a digital input chain by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the digital input chain, for instance
                MyDevice.inputChain.

        @return a YInputChain object allowing you to drive the digital input chain.
        """
        obj: Union[YInputChain, None]
        obj = YFunction._FindFromCacheInContext(yctx, "InputChain", func)
        if obj is None:
            obj = YInputChain(yctx, func)
            YFunction._AddToCache("InputChain", func, obj)
        return obj

    if not _IS_MICROPYTHON:
        async def registerValueCallback(self, callback: YInputChainValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerValueCallback(callback)

    async def resetWatchdog(self) -> int:
        """
        Resets the application watchdog countdown.
        If you have set up a non-zero watchdogPeriod, you should
        call this function on a regular basis to prevent the application
        inactivity error to be triggered.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.set_watchdogPeriod(-1)

    async def get_lastEvents(self) -> str:
        """
        Returns a string with last events observed on the digital input chain.
        This method return only events that are still buffered in the device memory.

        @return a string with last events observed (one per line).

        On failure, throws an exception or returns  YAPI.INVALID_STRING.
        """
        content: xarray

        content = await self._download("events.txt")
        return content.decode('latin-1')

    async def registerStateChangeCallback(self, callback: YStateChangeCallback) -> int:
        """
        Registers a callback function to be called each time that an event is detected on the
        input chain.The callback is invoked only during the execution of
        ySleep or yHandleEvents. This provides control over the time when
        the callback is triggered. For good responsiveness, remember to call one of these
        two functions periodically. To unregister a callback, pass a None pointer as argument.

        @param callback : the callback function to call, or a None pointer.
                The callback function should take four arguments:
                the YInputChain object that emitted the event, the
                UTC timestamp of the event, a character string describing
                the type of event and a character string with the event data.
                On failure, throws an exception or returns a negative error code.
        """
        if callback:
            await self.registerValueCallback(yInternalEventCallback)
        else:
            await self.registerValueCallback(None)
        # register user callback AFTER the internal pseudo-event,
        # to make sure we start with future events only
        self._stateChangeCallback = callback
        return 0

    async def _internalEventHandler(self, cbpos: str) -> int:
        newPos: int
        url: str
        content: xarray
        contentStr: str
        eventArr: list[str] = []
        arrLen: int
        lenStr: str
        arrPos: int
        eventStr: str
        eventLen: int
        hexStamp: str
        typePos: int
        dataPos: int
        evtStamp: int
        evtType: str
        evtData: str
        evtChange: str
        chainIdx: int
        newPos = YAPI._atoi(cbpos)
        if newPos < self._prevPos:
            self._eventPos = 0
        self._prevPos = newPos
        if newPos < self._eventPos:
            return YAPI.SUCCESS
        if not (self._stateChangeCallback):
            # first simulated event, use it to initialize reference values
            self._eventPos = newPos
            del self._eventChains[:]
            self._eventChains.append(await self.get_bitChain1())
            self._eventChains.append(await self.get_bitChain2())
            self._eventChains.append(await self.get_bitChain3())
            self._eventChains.append(await self.get_bitChain4())
            self._eventChains.append(await self.get_bitChain5())
            self._eventChains.append(await self.get_bitChain6())
            self._eventChains.append(await self.get_bitChain7())
            return YAPI.SUCCESS
        url = "events.txt?pos=%d" % self._eventPos

        content = await self._download(url)
        contentStr = content.decode('latin-1')
        eventArr = (contentStr).split('\n')
        arrLen = len(eventArr)
        if not (arrLen > 0):
            self._throw(YAPI.IO_ERROR, "fail to download events")
            return YAPI.IO_ERROR
        # last element of array is the new position preceeded by '@'
        arrLen = arrLen - 1
        lenStr = eventArr[arrLen]
        lenStr = lenStr[1: 1 + len(lenStr)-1]
        # update processed event position pointer
        self._eventPos = YAPI._atoi(lenStr)
        # now generate callbacks for each event received
        arrPos = 0
        while arrPos < arrLen:
            eventStr = eventArr[arrPos]
            eventLen = len(eventStr)
            if eventLen >= 1:
                hexStamp = eventStr[0: 0 + 8]
                evtStamp = int(hexStamp, 16)
                typePos = eventStr.find(":")+1
                if (evtStamp >= self._eventStamp) and(typePos > 8):
                    self._eventStamp = evtStamp
                    dataPos = eventStr.find("=")+1
                    evtType = eventStr[typePos: typePos + 1]
                    evtData = ""
                    evtChange = ""
                    if dataPos > 10:
                        evtData = eventStr[dataPos: dataPos + len(eventStr)-dataPos]
                        if ("1234567").find(evtType) >= 0:
                            chainIdx = YAPI._atoi(evtType) - 1
                            evtChange = await self._strXor(evtData, self._eventChains[chainIdx])
                            self._eventChains[chainIdx] = evtData
                    try:
                        retval = self._stateChangeCallback(self, evtStamp, evtType, evtData, evtChange)
                        if retval is not None: await retval
                    # noinspection PyBroadException
                    except Exception as e:
                        print('Exception in %s.stateChangeCallback:' % type(self).__name__, type(e).__name__, e)
            arrPos = arrPos + 1
        return YAPI.SUCCESS

    async def _strXor(self, a: str, b: str) -> str:
        lenA: int
        lenB: int
        res: str
        idx: int
        digitA: int
        digitB: int
        # make sure the result has the same length as first argument
        lenA = len(a)
        lenB = len(b)
        if lenA > lenB:
            res = a[0: 0 + lenA-lenB]
            a = a[lenA-lenB: lenA-lenB + lenB]
            lenA = lenB
        else:
            res = ""
            b = b[lenA-lenB: lenA-lenB + lenA]
        # scan strings and compare digit by digit
        idx = 0
        while idx < lenA:
            digitA = int(a[idx: idx + 1], 16)
            digitB = int(b[idx: idx + 1], 16)
            res = "%s%x" % (res, (digitA ^ digitB))
            idx = idx + 1
        return res

    async def hex2array(self, hexstr: str) -> list[int]:
        hexlen: int
        res: list[int] = []
        idx: int
        digit: int
        hexlen = len(hexstr)
        del res[:]

        idx = hexlen
        while idx > 0:
            idx = idx - 1
            digit = int(hexstr[idx: idx + 1], 16)
            res.append((digit & 1))
            res.append(((digit >> 1) & 1))
            res.append(((digit >> 2) & 1))
            res.append(((digit >> 3) & 1))

        return res

    # --- (end of YInputChain implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YStepperMotor
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YStepperMotor
version: PATCH_WITH_VERSION
requires: yocto_steppermotor_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_steppermotor_aio import YStepperMotor as YStepperMotor_aio
from .yocto_api import (
    YAPIContext, YAPI, YFunction
)

# --- (YStepperMotor class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YStepperMotorValueCallback = Union[Callable[['YStepperMotor', str], Awaitable[None]], None]
    except TypeError:
        YStepperMotorValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YStepperMotor(YFunction):
    """
    The YStepperMotor class allows you to drive a stepper motor.

    """
    _aio: YStepperMotor_aio
    # --- (end of YStepperMotor class start)
    if not _IS_MICROPYTHON:
        # --- (YStepperMotor return codes)
        DIAGS_INVALID: Final[int] = YAPI.INVALID_UINT
        STEPPOS_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        SPEED_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        PULLINSPEED_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        MAXACCEL_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        MAXSPEED_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        OVERCURRENT_INVALID: Final[int] = YAPI.INVALID_UINT
        TCURRSTOP_INVALID: Final[int] = YAPI.INVALID_UINT
        TCURRRUN_INVALID: Final[int] = YAPI.INVALID_UINT
        ALERTMODE_INVALID: Final[str] = YAPI.INVALID_STRING
        AUXMODE_INVALID: Final[str] = YAPI.INVALID_STRING
        AUXSIGNAL_INVALID: Final[int] = YAPI.INVALID_INT
        COMMAND_INVALID: Final[str] = YAPI.INVALID_STRING
        MOTORSTATE_ABSENT: Final[int] = 0
        MOTORSTATE_ALERT: Final[int] = 1
        MOTORSTATE_HI_Z: Final[int] = 2
        MOTORSTATE_STOP: Final[int] = 3
        MOTORSTATE_RUN: Final[int] = 4
        MOTORSTATE_BATCH: Final[int] = 5
        MOTORSTATE_INVALID: Final[int] = -1
        STEPPING_MICROSTEP16: Final[int] = 0
        STEPPING_MICROSTEP8: Final[int] = 1
        STEPPING_MICROSTEP4: Final[int] = 2
        STEPPING_HALFSTEP: Final[int] = 3
        STEPPING_FULLSTEP: Final[int] = 4
        STEPPING_INVALID: Final[int] = -1
        # --- (end of YStepperMotor return codes)


    # --- (YStepperMotor implementation)

    @classmethod
    def FirstStepperMotor(cls) -> Union[YStepperMotor, None]:
        """
        Starts the enumeration of stepper motors currently accessible.
        Use the method YStepperMotor.nextStepperMotor() to iterate on
        next stepper motors.

        @return a pointer to a YStepperMotor object, corresponding to
                the first stepper motor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YStepperMotor_aio.FirstStepperMotor())

    @classmethod
    def FirstStepperMotorInContext(cls, yctx: YAPIContext) -> Union[YStepperMotor, None]:
        """
        Starts the enumeration of stepper motors currently accessible.
        Use the method YStepperMotor.nextStepperMotor() to iterate on
        next stepper motors.

        @param yctx : a YAPI context.

        @return a pointer to a YStepperMotor object, corresponding to
                the first stepper motor currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YStepperMotor_aio.FirstStepperMotorInContext(yctx))

    def nextStepperMotor(self):
        """
        Continues the enumeration of stepper motors started using yFirstStepperMotor().
        Caution: You can't make any assumption about the returned stepper motors order.
        If you want to find a specific a stepper motor, use StepperMotor.findStepperMotor()
        and a hardwareID or a logical name.

        @return a pointer to a YStepperMotor object, corresponding to
                a stepper motor currently online, or a None pointer
                if there are no more stepper motors to enumerate.
        """
        return self._proxy(type(self), self._aio.nextStepperMotor())

    if not _DYNAMIC_HELPERS:
        def get_motorState(self) -> int:
            """
            Returns the motor working state.

            @return a value among YStepperMotor.MOTORSTATE_ABSENT, YStepperMotor.MOTORSTATE_ALERT,
            YStepperMotor.MOTORSTATE_HI_Z, YStepperMotor.MOTORSTATE_STOP, YStepperMotor.MOTORSTATE_RUN and
            YStepperMotor.MOTORSTATE_BATCH corresponding to the motor working state

            On failure, throws an exception or returns YStepperMotor.MOTORSTATE_INVALID.
            """
            return self._run(self._aio.get_motorState())

    if not _DYNAMIC_HELPERS:
        def get_diags(self) -> int:
            """
            Returns the stepper motor controller diagnostics, as a bitmap.

            @return an integer corresponding to the stepper motor controller diagnostics, as a bitmap

            On failure, throws an exception or returns YStepperMotor.DIAGS_INVALID.
            """
            return self._run(self._aio.get_diags())

    if not _DYNAMIC_HELPERS:
        def set_stepPos(self, newval: float) -> int:
            """
            Changes the current logical motor position, measured in steps.
            This command does not cause any motor move, as its purpose is only to set up
            the origin of the position counter. The fractional part of the position,
            that corresponds to the physical position of the rotor, is not changed.
            To trigger a motor move, use methods moveTo() or moveRel()
            instead.

            @param newval : a floating point number corresponding to the current logical motor position, measured in steps

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_stepPos(newval))

    if not _DYNAMIC_HELPERS:
        def get_stepPos(self) -> float:
            """
            Returns the current logical motor position, measured in steps.
            The value may include a fractional part when micro-stepping is in use.

            @return a floating point number corresponding to the current logical motor position, measured in steps

            On failure, throws an exception or returns YStepperMotor.STEPPOS_INVALID.
            """
            return self._run(self._aio.get_stepPos())

    if not _DYNAMIC_HELPERS:
        def get_speed(self) -> float:
            """
            Returns current motor speed, measured in steps per second.
            To change speed, use method changeSpeed().

            @return a floating point number corresponding to current motor speed, measured in steps per second

            On failure, throws an exception or returns YStepperMotor.SPEED_INVALID.
            """
            return self._run(self._aio.get_speed())

    if not _DYNAMIC_HELPERS:
        def set_pullinSpeed(self, newval: float) -> int:
            """
            Changes the motor speed immediately reachable from stop state, measured in steps per second.

            @param newval : a floating point number corresponding to the motor speed immediately reachable from
            stop state, measured in steps per second

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_pullinSpeed(newval))

    if not _DYNAMIC_HELPERS:
        def get_pullinSpeed(self) -> float:
            """
            Returns the motor speed immediately reachable from stop state, measured in steps per second.

            @return a floating point number corresponding to the motor speed immediately reachable from stop
            state, measured in steps per second

            On failure, throws an exception or returns YStepperMotor.PULLINSPEED_INVALID.
            """
            return self._run(self._aio.get_pullinSpeed())

    if not _DYNAMIC_HELPERS:
        def set_maxAccel(self, newval: float) -> int:
            """
            Changes the maximal motor acceleration, measured in steps per second^2.

            @param newval : a floating point number corresponding to the maximal motor acceleration, measured
            in steps per second^2

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_maxAccel(newval))

    if not _DYNAMIC_HELPERS:
        def get_maxAccel(self) -> float:
            """
            Returns the maximal motor acceleration, measured in steps per second^2.

            @return a floating point number corresponding to the maximal motor acceleration, measured in steps per second^2

            On failure, throws an exception or returns YStepperMotor.MAXACCEL_INVALID.
            """
            return self._run(self._aio.get_maxAccel())

    if not _DYNAMIC_HELPERS:
        def set_maxSpeed(self, newval: float) -> int:
            """
            Changes the maximal motor speed, measured in steps per second.

            @param newval : a floating point number corresponding to the maximal motor speed, measured in steps per second

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_maxSpeed(newval))

    if not _DYNAMIC_HELPERS:
        def get_maxSpeed(self) -> float:
            """
            Returns the maximal motor speed, measured in steps per second.

            @return a floating point number corresponding to the maximal motor speed, measured in steps per second

            On failure, throws an exception or returns YStepperMotor.MAXSPEED_INVALID.
            """
            return self._run(self._aio.get_maxSpeed())

    if not _DYNAMIC_HELPERS:
        def get_stepping(self) -> int:
            """
            Returns the stepping mode used to drive the motor.

            @return a value among YStepperMotor.STEPPING_MICROSTEP16, YStepperMotor.STEPPING_MICROSTEP8,
            YStepperMotor.STEPPING_MICROSTEP4, YStepperMotor.STEPPING_HALFSTEP and
            YStepperMotor.STEPPING_FULLSTEP corresponding to the stepping mode used to drive the motor

            On failure, throws an exception or returns YStepperMotor.STEPPING_INVALID.
            """
            return self._run(self._aio.get_stepping())

    if not _DYNAMIC_HELPERS:
        def set_stepping(self, newval: int) -> int:
            """
            Changes the stepping mode used to drive the motor.

            @param newval : a value among YStepperMotor.STEPPING_MICROSTEP16,
            YStepperMotor.STEPPING_MICROSTEP8, YStepperMotor.STEPPING_MICROSTEP4,
            YStepperMotor.STEPPING_HALFSTEP and YStepperMotor.STEPPING_FULLSTEP corresponding to the stepping
            mode used to drive the motor

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_stepping(newval))

    if not _DYNAMIC_HELPERS:
        def get_overcurrent(self) -> int:
            """
            Returns the overcurrent alert and emergency stop threshold, measured in mA.

            @return an integer corresponding to the overcurrent alert and emergency stop threshold, measured in mA

            On failure, throws an exception or returns YStepperMotor.OVERCURRENT_INVALID.
            """
            return self._run(self._aio.get_overcurrent())

    if not _DYNAMIC_HELPERS:
        def set_overcurrent(self, newval: int) -> int:
            """
            Changes the overcurrent alert and emergency stop threshold, measured in mA.

            @param newval : an integer corresponding to the overcurrent alert and emergency stop threshold, measured in mA

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_overcurrent(newval))

    if not _DYNAMIC_HELPERS:
        def get_tCurrStop(self) -> int:
            """
            Returns the torque regulation current when the motor is stopped, measured in mA.

            @return an integer corresponding to the torque regulation current when the motor is stopped, measured in mA

            On failure, throws an exception or returns YStepperMotor.TCURRSTOP_INVALID.
            """
            return self._run(self._aio.get_tCurrStop())

    if not _DYNAMIC_HELPERS:
        def set_tCurrStop(self, newval: int) -> int:
            """
            Changes the torque regulation current when the motor is stopped, measured in mA.

            @param newval : an integer corresponding to the torque regulation current when the motor is
            stopped, measured in mA

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_tCurrStop(newval))

    if not _DYNAMIC_HELPERS:
        def get_tCurrRun(self) -> int:
            """
            Returns the torque regulation current when the motor is running, measured in mA.

            @return an integer corresponding to the torque regulation current when the motor is running, measured in mA

            On failure, throws an exception or returns YStepperMotor.TCURRRUN_INVALID.
            """
            return self._run(self._aio.get_tCurrRun())

    if not _DYNAMIC_HELPERS:
        def set_tCurrRun(self, newval: int) -> int:
            """
            Changes the torque regulation current when the motor is running, measured in mA.

            @param newval : an integer corresponding to the torque regulation current when the motor is
            running, measured in mA

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_tCurrRun(newval))

    if not _DYNAMIC_HELPERS:
        def set_alertMode(self, newval: str) -> int:
            return self._run(self._aio.set_alertMode(newval))

    if not _DYNAMIC_HELPERS:
        def set_auxMode(self, newval: str) -> int:
            return self._run(self._aio.set_auxMode(newval))

    if not _DYNAMIC_HELPERS:
        def get_auxSignal(self) -> int:
            """
            Returns the current value of the signal generated on the auxiliary output.

            @return an integer corresponding to the current value of the signal generated on the auxiliary output

            On failure, throws an exception or returns YStepperMotor.AUXSIGNAL_INVALID.
            """
            return self._run(self._aio.get_auxSignal())

    if not _DYNAMIC_HELPERS:
        def set_auxSignal(self, newval: int) -> int:
            """
            Changes the value of the signal generated on the auxiliary output.
            Acceptable values depend on the auxiliary output signal type configured.

            @param newval : an integer corresponding to the value of the signal generated on the auxiliary output

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_auxSignal(newval))

    if not _DYNAMIC_HELPERS:
        def set_command(self, newval: str) -> int:
            return self._run(self._aio.set_command(newval))

    @classmethod
    def FindStepperMotor(cls, func: str) -> YStepperMotor:
        """
        Retrieves a stepper motor for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the stepper motor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YStepperMotor.isOnline() to test if the stepper motor is
        indeed online at a given time. In case of ambiguity when looking for
        a stepper motor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the stepper motor, for instance
                MyDevice.stepperMotor1.

        @return a YStepperMotor object allowing you to drive the stepper motor.
        """
        return cls._proxy(cls, YStepperMotor_aio.FindStepperMotor(func))

    @classmethod
    def FindStepperMotorInContext(cls, yctx: YAPIContext, func: str) -> YStepperMotor:
        """
        Retrieves a stepper motor for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the stepper motor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YStepperMotor.isOnline() to test if the stepper motor is
        indeed online at a given time. In case of ambiguity when looking for
        a stepper motor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the stepper motor, for instance
                MyDevice.stepperMotor1.

        @return a YStepperMotor object allowing you to drive the stepper motor.
        """
        return cls._proxy(cls, YStepperMotor_aio.FindStepperMotorInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YStepperMotorValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _DYNAMIC_HELPERS:
        def reset(self) -> int:
            """
            Reinitialize the controller and clear all alert flags.

            @return YAPI.SUCCESS if the call succeeds.
                    On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.reset())

    if not _DYNAMIC_HELPERS:
        def findHomePosition(self, speed: float) -> int:
            """
            Starts the motor backward at the specified speed, to search for the motor home position.

            @param speed : desired speed, in steps per second.

            @return YAPI.SUCCESS if the call succeeds.
                    On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.findHomePosition(speed))

    if not _DYNAMIC_HELPERS:
        def changeSpeed(self, speed: float) -> int:
            """
            Starts the motor at a given speed. The time needed to reach the requested speed
            will depend on the acceleration parameters configured for the motor.

            @param speed : desired speed, in steps per second. The minimal non-zero speed
                    is 0.001 pulse per second.

            @return YAPI.SUCCESS if the call succeeds.
                    On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.changeSpeed(speed))

    if not _DYNAMIC_HELPERS:
        def moveTo(self, absPos: float) -> int:
            """
            Starts the motor to reach a given absolute position. The time needed to reach the requested
            position will depend on the acceleration and max speed parameters configured for
            the motor.

            @param absPos : absolute position, measured in steps from the origin.

            @return YAPI.SUCCESS if the call succeeds.
                    On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.moveTo(absPos))

    if not _DYNAMIC_HELPERS:
        def moveRel(self, relPos: float) -> int:
            """
            Starts the motor to reach a given relative position. The time needed to reach the requested
            position will depend on the acceleration and max speed parameters configured for
            the motor.

            @param relPos : relative position, measured in steps from the current position.

            @return YAPI.SUCCESS if the call succeeds.
                    On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.moveRel(relPos))

    if not _DYNAMIC_HELPERS:
        def moveRelSlow(self, relPos: float, maxSpeed: float) -> int:
            """
            Starts the motor to reach a given relative position, keeping the speed under the
            specified limit. The time needed to reach the requested position will depend on
            the acceleration parameters configured for the motor.

            @param relPos : relative position, measured in steps from the current position.
            @param maxSpeed : limit speed, in steps per second.

            @return YAPI.SUCCESS if the call succeeds.
                    On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.moveRelSlow(relPos, maxSpeed))

    if not _DYNAMIC_HELPERS:
        def pause(self, waitMs: int) -> int:
            """
            Keep the motor in the same state for the specified amount of time, before processing next command.

            @param waitMs : wait time, specified in milliseconds.

            @return YAPI.SUCCESS if the call succeeds.
                    On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.pause(waitMs))

    if not _DYNAMIC_HELPERS:
        def emergencyStop(self) -> int:
            """
            Stops the motor with an emergency alert, without taking any additional precaution.

            @return YAPI.SUCCESS if the call succeeds.
                    On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.emergencyStop())

    if not _DYNAMIC_HELPERS:
        def alertStepOut(self) -> int:
            """
            Move one step in the direction opposite the direction set when the most recent alert was raised.
            The move occurs even if the system is still in alert mode (end switch depressed). Caution.
            use this function with great care as it may cause mechanical damages !

            @return YAPI.SUCCESS if the call succeeds.
                    On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.alertStepOut())

    if not _DYNAMIC_HELPERS:
        def alertStepDir(self, dir: int) -> int:
            """
            Move one single step in the selected direction without regards to end switches.
            The move occurs even if the system is still in alert mode (end switch depressed). Caution.
            use this function with great care as it may cause mechanical damages !

            @param dir : Value +1 or -1, according to the desired direction of the move

            @return YAPI.SUCCESS if the call succeeds.
                    On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.alertStepDir(dir))

    if not _DYNAMIC_HELPERS:
        def abortAndBrake(self) -> int:
            """
            Stops the motor smoothly as soon as possible, without waiting for ongoing move completion.

            @return YAPI.SUCCESS if the call succeeds.
                    On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.abortAndBrake())

    if not _DYNAMIC_HELPERS:
        def abortAndHiZ(self) -> int:
            """
            Turn the controller into Hi-Z mode immediately, without waiting for ongoing move completion.

            @return YAPI.SUCCESS if the call succeeds.
                    On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.abortAndHiZ())

    # --- (end of YStepperMotor implementation)




#!/usr/bin/python
# ********************************************************************
#
#  Example PyQt6 GUI application using Yoctopuce devices
#
#  This code shows how to properly use a separate thread
#  for running possibly blocking I/O operations
#
#  For more details, read
#  https://www.yoctopuce.com/EN/article/creating-a-yoctopuce-application-with-pyqt
#
# *********************************************************************
# -*- coding: utf-8 -*-
import os, sys,time

# add ../../Sources to the PYTHONPATH
sys.path.append(os.path.join("..", "..", "Sources"))
from yocto_anbutton import *
from yocto_relay import YRelay

from PyQt6.QtWidgets import QApplication, QWidget, QMainWindow, QVBoxLayout, QLabel, QPushButton
from PyQt6.QtCore import pyqtSlot, pyqtSignal, QObject, QThread, QTimer

# Starting with PyQt v5.5, unhandled Python exception result in a call
# to Qt’s qFatal() function, which will cause your application to stop
# without any explanation in an IDE like PyCharm, and prevent debugging.
# The lines below restore the previous behaviour
def except_hook(cls, exception, traceback):
    sys.__excepthook__(cls, exception, traceback)
sys.excepthook = except_hook

# Helper to get the unique identifier of current thread
def currThread():
    return '[thread-' + str(int(QThread.currentThreadId())) + ']'

class YoctopuceTask(QObject):

    startTask = pyqtSignal()            # in: start the task
    stopTask = pyqtSignal()             # in: stop the task
    toggleRelay = pyqtSignal(str)       # in: toggle a relay
    statusMsg = pyqtSignal(str)         # out: publish the task status
    arrival = pyqtSignal(dict)          # out: publish a new device arrival
    newValue = pyqtSignal(str,str)      # out: publish a new function value
    removal = pyqtSignal(dict)          # out: publish a device disconnect

    def __init__(self):
        super(YoctopuceTask, self).__init__()
        # connect incoming signals
        self.startTask.connect(self.initAPI)
        self.toggleRelay.connect(self.toggleIt)
        self.stopTask.connect(self.freeAPI)

    @pyqtSlot()
    def initAPI(self):
        print('Yoctopuce task started', currThread())
        errmsg = YRefParam()
        YAPI.RegisterLogFunction(self.logfun)
        # Setup the API to use Yoctopuce devices on localhost
        if YAPI.RegisterHub('127.0.0.1', errmsg) != YAPI.SUCCESS:
            self.statusMsg.emit('Failed to init Yoctopuce API: ' +
                                errmsg.value)
            return
        YAPI.RegisterDeviceArrivalCallback(self.deviceArrival)
        YAPI.RegisterDeviceRemovalCallback(self.deviceRemoval)
        self.statusMsg.emit('Yoctopuce task ready')
        # prepare to scan Yoctopuce events periodically
        self.timer = QTimer()
        self.timer.timeout.connect(self.handleEvents)
        self.checkDevices = 0
        self.timer.start(50)

    @pyqtSlot()
    def freeAPI(self):
        self.timer.stop()
        YAPI.FreeAPI()
        self.statusMsg.emit('Yoctopuce task stopped')

    @pyqtSlot()
    def handleEvents(self):
        if self.checkDevices <= 0:
            YAPI.UpdateDeviceList()
            self.checkDevices = 10
        else:
            self.checkDevices -= 1
        YAPI.HandleEvents()

    @pyqtSlot(str)
    def toggleIt(self, relayName):
        print("Toggle "+relayName, currThread())
        relay = YRelay.FindRelay(relayName)
        if relay.isOnline():
            relay.toggle()
        else:
            self.statusMsg.emit('Relay "' + relayName + '" is not online')

    def deviceArrival(self, m: YModule):
        serialNumber = m.get_serialNumber()
        # build a description of the device as a dictionnary
        device = { 'serial': serialNumber, 'functions': {} }
        fctcount = m.functionCount()
        for i in range(fctcount):
            device['functions'][m.functionId(i)] = m.functionType(i)
        m.set_userData(device)
        # pass it to the UI thread via the arrival signal
        self.arrival.emit(device)
        # make sure to get notified about each new value
        for functionId in device['functions']:
            bt = YFunction.FindFunction(serialNumber + '.' + functionId)
            bt.registerValueCallback(self.functionValueChangeCallback)

    def functionValueChangeCallback(self, fct: YFunction, value: str):
        hardwareId = fct.get_hardwareId()
        self.newValue.emit(hardwareId, value)

    def deviceRemoval(self, m: YModule):
        # pass the disconnect to the UI thread via the removal signal
        self.removal.emit(m.get_userData())

    def logfun(self, line: str):
        msg = line.rstrip()
        print("API Log: " + msg, currThread())
        # show low-level API logs as status
        self.statusMsg.emit(msg)

class SensorDisplay(QMainWindow):
    def __init__(self):
        super().__init__()
        self.refreshCount = 0
        self.setWindowTitle('Sample PyQT6 application')
        self.message = QLabel(self)
        self.message.setStyleSheet("background: white")
        self.functionValues = {}
        self.layout = QVBoxLayout()
        self.layout.addWidget(self.message)
        widget = QWidget()
        widget.setLayout(self.layout)
        self.setCentralWidget(widget)
        self.show()
        print("MainWindow created", currThread())

    def startIO(self):
        # Start Yoctopuce I/O task in a separate thread
        self.yoctoThread = QThread()
        self.yoctoThread.start()
        self.yoctoTask = YoctopuceTask()
        self.yoctoTask.moveToThread(self.yoctoThread)
        self.yoctoTask.statusMsg.connect(self.showMsg)
        self.yoctoTask.arrival.connect(self.arrival)
        self.yoctoTask.newValue.connect(self.newValue)
        self.yoctoTask.removal.connect(self.removal)
        self.yoctoTask.startTask.emit()

    @pyqtSlot(str)
    def showMsg(self, str):
        self.message.setText(str)

    @pyqtSlot(dict)
    def arrival(self, device):
        # log arrival
        print('Device connected:', device, currThread())
        # for relay functions, create a toggle button
        for functionId in device['functions']:
            if device['functions'][functionId] == 'Relay':
                hardwareId = device['serial'] + '.' + functionId
                newButton = QPushButton('toggle '+hardwareId)
                newButton.clicked.connect(
                    lambda checked,hwid=hardwareId:
                        self.toggleRelayPressed(hwid)
                )
                self.layout.addWidget(newButton)

    @pyqtSlot(str,str)
    def newValue(self, hardwareId, value):
        if hardwareId not in self.functionValues:
            # create a new label when first value arrives
            newLabel = QLabel(self)
            self.layout.addWidget(newLabel)
            self.functionValues[hardwareId] = newLabel
        # then update it for each reported value
        self.functionValues[hardwareId].setText(hardwareId + ": " + value)

    @pyqtSlot(QPushButton)
    def toggleRelayPressed(self, hardwareId):
        print('toggleRelayPressed:', hardwareId, currThread())
        self.yoctoTask.toggleRelay.emit(hardwareId)

    @pyqtSlot(dict)
    def removal(self, device):
        # log arrival
        print('Device disconnected:', device, currThread())
        # mark all reported values as disconnected
        for functionId in device['functions']:
            hardwareId = device['serial'] + '.' + functionId
            if hardwareId in self.functionValues:
                self.functionValues[hardwareId].setText(hardwareId + ": disconnected")

app = QApplication(sys.argv)
window = SensorDisplay()
window.startIO()
sys.exit(app.exec())



# ********************************************************************
#
#  $Id: helloworld.py 66265 2025-05-06 06:51:11Z seb $
#
#  An example that show how to use a  Yocto-CO2
#
#  You can find more information on our web site:
#   Yocto-CO2 documentation:
#      https://www.yoctopuce.com/EN/products/yocto-co2/doc.html
#   Python API Reference:
#      https://www.yoctopuce.com/EN/doc/reference/yoctolib-python-EN.html
#
# *********************************************************************
import sys

from yoctolib.yocto_api import YRefParam, YAPI
from yoctolib.yocto_voc import YVoc


def die(msg):
    YAPI.FreeAPI()
    sys.exit(msg + ' (check USB cable)')


# the API use local USB devices through VirtualHub
errmsg = YRefParam()
if YAPI.RegisterHub("localhost", errmsg) != YAPI.SUCCESS:
    sys.exit("RegisterHub failed: " + errmsg.value)

# To use a specific device, invoke the script as
#   python helloworld.py [serial_number]
# or
#   python helloworld.py [logical_name]
target = 'any'
if len(sys.argv) > 1:
    target = sys.argv[1]

if target == 'any':
    # retrieve any voc sensor
    sensor = YVoc.FirstVoc()
    if sensor is None:
        die('No Yocto-VOC connected')
    target = sensor.get_serialNumber()

# retrieve specified functions
sensor = YVoc.FindVoc(target + '.voc')
if not (sensor.isOnline()):
    die('device not connected')

print("Use device %s" % sensor.get_serialNumber())
while sensor.isOnline():
    print("VOC :  " + "%2.1f" % sensor.get_currentValue() + "ppm (Ctrl-C to stop)")
    YAPI.Sleep(1000)

YAPI.FreeAPI()



# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  Asyncio implementation of YAccelerometer
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: Asyncio implementation of YAccelerometer
version: PATCH_WITH_VERSION
requires: yocto_api_aio
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api_aio import const, _IS_MICROPYTHON
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa

from .yocto_api_aio import (
    YAPIContext, YAPI, YAPI_Exception, HwId, hwid2str, YFunction, YSensor, YMeasure
)

# --- (YAccelerometer class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YAccelerometerValueCallback = Union[Callable[['YAccelerometer', str], Awaitable[None]], None]
        YAccelerometerTimedReportCallback = Union[Callable[['YAccelerometer', YMeasure], Awaitable[None]], None]
    except TypeError:
        YAccelerometerValueCallback = Union[Callable, Awaitable]
        YAccelerometerTimedReportCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YAccelerometer(YSensor):
    """
    The YAccelerometer class allows you to read and configure Yoctopuce accelerometers.
    It inherits from YSensor class the core functions to read measurements,
    to register callback functions, and to access the autonomous datalogger.
    This class adds the possibility to access x, y and z components of the acceleration
    vector separately.

    """
    # --- (end of YAccelerometer class start)
    if not _IS_MICROPYTHON:
        # --- (YAccelerometer return codes)
        BANDWIDTH_INVALID: Final[int] = YAPI.INVALID_UINT
        XVALUE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        YVALUE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        ZVALUE_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        GRAVITYCANCELLATION_OFF: Final[int] = 0
        GRAVITYCANCELLATION_ON: Final[int] = 1
        GRAVITYCANCELLATION_INVALID: Final[int] = -1
        # --- (end of YAccelerometer return codes)

    # --- (YAccelerometer attributes declaration)
    _bandwidth: int
    _xValue: float
    _yValue: float
    _zValue: float
    _gravityCancellation: int
    _valueCallback: YAccelerometerValueCallback
    _timedReportCallback: YAccelerometerTimedReportCallback
    # --- (end of YAccelerometer attributes declaration)


    def __init__(self, yctx: YAPIContext, func: str):
        super().__init__(yctx, func)
        self._className = 'Accelerometer'
        # --- (YAccelerometer constructor)
        self._bandwidth = YAccelerometer.BANDWIDTH_INVALID
        self._xValue = YAccelerometer.XVALUE_INVALID
        self._yValue = YAccelerometer.YVALUE_INVALID
        self._zValue = YAccelerometer.ZVALUE_INVALID
        self._gravityCancellation = YAccelerometer.GRAVITYCANCELLATION_INVALID
        # --- (end of YAccelerometer constructor)

    # --- (YAccelerometer implementation)

    @staticmethod
    def FirstAccelerometer() -> Union[YAccelerometer, None]:
        """
        Starts the enumeration of accelerometers currently accessible.
        Use the method YAccelerometer.nextAccelerometer() to iterate on
        next accelerometers.

        @return a pointer to a YAccelerometer object, corresponding to
                the first accelerometer currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = YAPI._yHash.getFirstHardwareId('Accelerometer')
        if not next_hwid:
            return None
        return YAccelerometer.FindAccelerometer(hwid2str(next_hwid))

    @staticmethod
    def FirstAccelerometerInContext(yctx: YAPIContext) -> Union[YAccelerometer, None]:
        """
        Starts the enumeration of accelerometers currently accessible.
        Use the method YAccelerometer.nextAccelerometer() to iterate on
        next accelerometers.

        @param yctx : a YAPI context.

        @return a pointer to a YAccelerometer object, corresponding to
                the first accelerometer currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = yctx._yHash.getFirstHardwareId('Accelerometer')
        if not next_hwid:
            return None
        return YAccelerometer.FindAccelerometerInContext(yctx, hwid2str(next_hwid))

    def nextAccelerometer(self):
        """
        Continues the enumeration of accelerometers started using yFirstAccelerometer().
        Caution: You can't make any assumption about the returned accelerometers order.
        If you want to find a specific an accelerometer, use Accelerometer.findAccelerometer()
        and a hardwareID or a logical name.

        @return a pointer to a YAccelerometer object, corresponding to
                an accelerometer currently online, or a None pointer
                if there are no more accelerometers to enumerate.
        """
        next_hwid: Union[HwId, None] = None
        try:
            hwid: HwId = self._yapi._yHash.resolveHwID(self._className, self._func)
            next_hwid = self._yapi._yHash.getNextHardwareId(self._className, hwid)
        except YAPI_Exception:
            pass
        if not next_hwid:
            return None
        return YAccelerometer.FindAccelerometerInContext(self._yapi, hwid2str(next_hwid))

    def _parseAttr(self, json_val: dict) -> None:
        if 'bandwidth' in json_val:
            self._bandwidth = json_val["bandwidth"]
        if 'xValue' in json_val:
            self._xValue = round(json_val["xValue"] / 65.536) / 1000.0
        if 'yValue' in json_val:
            self._yValue = round(json_val["yValue"] / 65.536) / 1000.0
        if 'zValue' in json_val:
            self._zValue = round(json_val["zValue"] / 65.536) / 1000.0
        if 'gravityCancellation' in json_val:
            self._gravityCancellation = json_val["gravityCancellation"] > 0
        super()._parseAttr(json_val)

    async def get_bandwidth(self) -> int:
        """
        Returns the measure update frequency, measured in Hz.

        @return an integer corresponding to the measure update frequency, measured in Hz

        On failure, throws an exception or returns YAccelerometer.BANDWIDTH_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YAccelerometer.BANDWIDTH_INVALID
        res = self._bandwidth
        return res

    async def set_bandwidth(self, newval: int) -> int:
        """
        Changes the measure update frequency, measured in Hz. When the
        frequency is lower, the device performs averaging.
        Remember to call the saveToFlash()
        method of the module if the modification must be kept.

        @param newval : an integer corresponding to the measure update frequency, measured in Hz

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("bandwidth", rest_val)

    async def get_xValue(self) -> float:
        """
        Returns the X component of the acceleration, as a floating point number.

        @return a floating point number corresponding to the X component of the acceleration, as a floating point number

        On failure, throws an exception or returns YAccelerometer.XVALUE_INVALID.
        """
        res: float
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YAccelerometer.XVALUE_INVALID
        res = self._xValue
        return res

    async def get_yValue(self) -> float:
        """
        Returns the Y component of the acceleration, as a floating point number.

        @return a floating point number corresponding to the Y component of the acceleration, as a floating point number

        On failure, throws an exception or returns YAccelerometer.YVALUE_INVALID.
        """
        res: float
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YAccelerometer.YVALUE_INVALID
        res = self._yValue
        return res

    async def get_zValue(self) -> float:
        """
        Returns the Z component of the acceleration, as a floating point number.

        @return a floating point number corresponding to the Z component of the acceleration, as a floating point number

        On failure, throws an exception or returns YAccelerometer.ZVALUE_INVALID.
        """
        res: float
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YAccelerometer.ZVALUE_INVALID
        res = self._zValue
        return res

    async def get_gravityCancellation(self) -> int:
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YAccelerometer.GRAVITYCANCELLATION_INVALID
        res = self._gravityCancellation
        return res

    async def set_gravityCancellation(self, newval: int) -> int:
        rest_val = "1" if newval > 0 else "0"
        return await self._setAttr("gravityCancellation", rest_val)

    @staticmethod
    def FindAccelerometer(func: str) -> YAccelerometer:
        """
        Retrieves an accelerometer for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the accelerometer is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YAccelerometer.isOnline() to test if the accelerometer is
        indeed online at a given time. In case of ambiguity when looking for
        an accelerometer by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the accelerometer, for instance
                Y3DMK002.accelerometer.

        @return a YAccelerometer object allowing you to drive the accelerometer.
        """
        obj: Union[YAccelerometer, None]
        obj = YFunction._FindFromCache("Accelerometer", func)
        if obj is None:
            obj = YAccelerometer(YAPI, func)
            YFunction._AddToCache("Accelerometer", func, obj)
        return obj

    @staticmethod
    def FindAccelerometerInContext(yctx: YAPIContext, func: str) -> YAccelerometer:
        """
        Retrieves an accelerometer for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the accelerometer is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YAccelerometer.isOnline() to test if the accelerometer is
        indeed online at a given time. In case of ambiguity when looking for
        an accelerometer by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the accelerometer, for instance
                Y3DMK002.accelerometer.

        @return a YAccelerometer object allowing you to drive the accelerometer.
        """
        obj: Union[YAccelerometer, None]
        obj = YFunction._FindFromCacheInContext(yctx, "Accelerometer", func)
        if obj is None:
            obj = YAccelerometer(yctx, func)
            YFunction._AddToCache("Accelerometer", func, obj)
        return obj

    if not _IS_MICROPYTHON:
        async def registerValueCallback(self, callback: YAccelerometerValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerValueCallback(callback)

    if not _IS_MICROPYTHON:
        async def registerTimedReportCallback(self, callback: YAccelerometerTimedReportCallback) -> int:
            """
            Registers the callback function that is invoked on every periodic timed notification.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and an YMeasure object describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerTimedReportCallback(callback)

    # --- (end of YAccelerometer implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YInputChain
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YInputChain
version: PATCH_WITH_VERSION
requires: yocto_inputchain_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_inputchain_aio import YInputChain as YInputChain_aio
from .yocto_api import (
    YAPIContext, YAPI, YFunction
)

# --- (YInputChain class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YInputChainValueCallback = Union[Callable[['YInputChain', str], Awaitable[None]], None]
        YStateChangeCallback = Union[Callable[['YInputChain',int , str, str, str], Awaitable[None]], None]
    except TypeError:
        YInputChainValueCallback = Union[Callable, Awaitable]
        YStateChangeCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YInputChain(YFunction):
    """
    The YInputChain class provides access to separate
    digital inputs connected in a chain.

    """
    _aio: YInputChain_aio
    # --- (end of YInputChain class start)
    if not _IS_MICROPYTHON:
        # --- (YInputChain return codes)
        EXPECTEDNODES_INVALID: Final[int] = YAPI.INVALID_UINT
        DETECTEDNODES_INVALID: Final[int] = YAPI.INVALID_UINT
        REFRESHRATE_INVALID: Final[int] = YAPI.INVALID_UINT
        BITCHAIN1_INVALID: Final[str] = YAPI.INVALID_STRING
        BITCHAIN2_INVALID: Final[str] = YAPI.INVALID_STRING
        BITCHAIN3_INVALID: Final[str] = YAPI.INVALID_STRING
        BITCHAIN4_INVALID: Final[str] = YAPI.INVALID_STRING
        BITCHAIN5_INVALID: Final[str] = YAPI.INVALID_STRING
        BITCHAIN6_INVALID: Final[str] = YAPI.INVALID_STRING
        BITCHAIN7_INVALID: Final[str] = YAPI.INVALID_STRING
        WATCHDOGPERIOD_INVALID: Final[int] = YAPI.INVALID_UINT
        CHAINDIAGS_INVALID: Final[int] = YAPI.INVALID_UINT
        LOOPBACKTEST_OFF: Final[int] = 0
        LOOPBACKTEST_ON: Final[int] = 1
        LOOPBACKTEST_INVALID: Final[int] = -1
        # --- (end of YInputChain return codes)


    # --- (YInputChain implementation)

    @classmethod
    def FirstInputChain(cls) -> Union[YInputChain, None]:
        """
        Starts the enumeration of digital input chains currently accessible.
        Use the method YInputChain.nextInputChain() to iterate on
        next digital input chains.

        @return a pointer to a YInputChain object, corresponding to
                the first digital input chain currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YInputChain_aio.FirstInputChain())

    @classmethod
    def FirstInputChainInContext(cls, yctx: YAPIContext) -> Union[YInputChain, None]:
        """
        Starts the enumeration of digital input chains currently accessible.
        Use the method YInputChain.nextInputChain() to iterate on
        next digital input chains.

        @param yctx : a YAPI context.

        @return a pointer to a YInputChain object, corresponding to
                the first digital input chain currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YInputChain_aio.FirstInputChainInContext(yctx))

    def nextInputChain(self):
        """
        Continues the enumeration of digital input chains started using yFirstInputChain().
        Caution: You can't make any assumption about the returned digital input chains order.
        If you want to find a specific a digital input chain, use InputChain.findInputChain()
        and a hardwareID or a logical name.

        @return a pointer to a YInputChain object, corresponding to
                a digital input chain currently online, or a None pointer
                if there are no more digital input chains to enumerate.
        """
        return self._proxy(type(self), self._aio.nextInputChain())

    if not _DYNAMIC_HELPERS:
        def get_expectedNodes(self) -> int:
            """
            Returns the number of nodes expected in the chain.

            @return an integer corresponding to the number of nodes expected in the chain

            On failure, throws an exception or returns YInputChain.EXPECTEDNODES_INVALID.
            """
            return self._run(self._aio.get_expectedNodes())

    if not _DYNAMIC_HELPERS:
        def set_expectedNodes(self, newval: int) -> int:
            """
            Changes the number of nodes expected in the chain.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : an integer corresponding to the number of nodes expected in the chain

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_expectedNodes(newval))

    if not _DYNAMIC_HELPERS:
        def get_detectedNodes(self) -> int:
            """
            Returns the number of nodes detected in the chain.

            @return an integer corresponding to the number of nodes detected in the chain

            On failure, throws an exception or returns YInputChain.DETECTEDNODES_INVALID.
            """
            return self._run(self._aio.get_detectedNodes())

    if not _DYNAMIC_HELPERS:
        def get_loopbackTest(self) -> int:
            """
            Returns the activation state of the exhaustive chain connectivity test.
            The connectivity test requires a cable connecting the end of the chain
            to the loopback test connector.

            @return either YInputChain.LOOPBACKTEST_OFF or YInputChain.LOOPBACKTEST_ON, according to the
            activation state of the exhaustive chain connectivity test

            On failure, throws an exception or returns YInputChain.LOOPBACKTEST_INVALID.
            """
            return self._run(self._aio.get_loopbackTest())

    if not _DYNAMIC_HELPERS:
        def set_loopbackTest(self, newval: int) -> int:
            """
            Changes the activation state of the exhaustive chain connectivity test.
            The connectivity test requires a cable connecting the end of the chain
            to the loopback test connector.

            If you want the change to be kept after a device reboot,
            make sure  to call the matching module saveToFlash().

            @param newval : either YInputChain.LOOPBACKTEST_OFF or YInputChain.LOOPBACKTEST_ON, according to
            the activation state of the exhaustive chain connectivity test

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_loopbackTest(newval))

    if not _DYNAMIC_HELPERS:
        def get_refreshRate(self) -> int:
            """
            Returns the desired refresh rate, measured in Hz.
            The higher the refresh rate is set, the higher the
            communication speed on the chain will be.

            @return an integer corresponding to the desired refresh rate, measured in Hz

            On failure, throws an exception or returns YInputChain.REFRESHRATE_INVALID.
            """
            return self._run(self._aio.get_refreshRate())

    if not _DYNAMIC_HELPERS:
        def set_refreshRate(self, newval: int) -> int:
            """
            Changes the desired refresh rate, measured in Hz.
            The higher the refresh rate is set, the higher the
            communication speed on the chain will be.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : an integer corresponding to the desired refresh rate, measured in Hz

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_refreshRate(newval))

    if not _DYNAMIC_HELPERS:
        def get_bitChain1(self) -> str:
            """
            Returns the state of input 1 for all nodes of the input chain,
            as a hexadecimal string. The node nearest to the controller
            is the lowest bit of the result.

            @return a string corresponding to the state of input 1 for all nodes of the input chain,
                    as a hexadecimal string

            On failure, throws an exception or returns YInputChain.BITCHAIN1_INVALID.
            """
            return self._run(self._aio.get_bitChain1())

    if not _DYNAMIC_HELPERS:
        def get_bitChain2(self) -> str:
            """
            Returns the state of input 2 for all nodes of the input chain,
            as a hexadecimal string. The node nearest to the controller
            is the lowest bit of the result.

            @return a string corresponding to the state of input 2 for all nodes of the input chain,
                    as a hexadecimal string

            On failure, throws an exception or returns YInputChain.BITCHAIN2_INVALID.
            """
            return self._run(self._aio.get_bitChain2())

    if not _DYNAMIC_HELPERS:
        def get_bitChain3(self) -> str:
            """
            Returns the state of input 3 for all nodes of the input chain,
            as a hexadecimal string. The node nearest to the controller
            is the lowest bit of the result.

            @return a string corresponding to the state of input 3 for all nodes of the input chain,
                    as a hexadecimal string

            On failure, throws an exception or returns YInputChain.BITCHAIN3_INVALID.
            """
            return self._run(self._aio.get_bitChain3())

    if not _DYNAMIC_HELPERS:
        def get_bitChain4(self) -> str:
            """
            Returns the state of input 4 for all nodes of the input chain,
            as a hexadecimal string. The node nearest to the controller
            is the lowest bit of the result.

            @return a string corresponding to the state of input 4 for all nodes of the input chain,
                    as a hexadecimal string

            On failure, throws an exception or returns YInputChain.BITCHAIN4_INVALID.
            """
            return self._run(self._aio.get_bitChain4())

    if not _DYNAMIC_HELPERS:
        def get_bitChain5(self) -> str:
            """
            Returns the state of input 5 for all nodes of the input chain,
            as a hexadecimal string. The node nearest to the controller
            is the lowest bit of the result.

            @return a string corresponding to the state of input 5 for all nodes of the input chain,
                    as a hexadecimal string

            On failure, throws an exception or returns YInputChain.BITCHAIN5_INVALID.
            """
            return self._run(self._aio.get_bitChain5())

    if not _DYNAMIC_HELPERS:
        def get_bitChain6(self) -> str:
            """
            Returns the state of input 6 for all nodes of the input chain,
            as a hexadecimal string. The node nearest to the controller
            is the lowest bit of the result.

            @return a string corresponding to the state of input 6 for all nodes of the input chain,
                    as a hexadecimal string

            On failure, throws an exception or returns YInputChain.BITCHAIN6_INVALID.
            """
            return self._run(self._aio.get_bitChain6())

    if not _DYNAMIC_HELPERS:
        def get_bitChain7(self) -> str:
            """
            Returns the state of input 7 for all nodes of the input chain,
            as a hexadecimal string. The node nearest to the controller
            is the lowest bit of the result.

            @return a string corresponding to the state of input 7 for all nodes of the input chain,
                    as a hexadecimal string

            On failure, throws an exception or returns YInputChain.BITCHAIN7_INVALID.
            """
            return self._run(self._aio.get_bitChain7())

    if not _DYNAMIC_HELPERS:
        def get_watchdogPeriod(self) -> int:
            """
            Returns the wait time in seconds before triggering an inactivity
            timeout error.

            @return an integer corresponding to the wait time in seconds before triggering an inactivity
                    timeout error

            On failure, throws an exception or returns YInputChain.WATCHDOGPERIOD_INVALID.
            """
            return self._run(self._aio.get_watchdogPeriod())

    if not _DYNAMIC_HELPERS:
        def set_watchdogPeriod(self, newval: int) -> int:
            """
            Changes the wait time in seconds before triggering an inactivity
            timeout error. Remember to call the saveToFlash() method
            of the module if the modification must be kept.

            @param newval : an integer corresponding to the wait time in seconds before triggering an inactivity
                    timeout error

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_watchdogPeriod(newval))

    if not _DYNAMIC_HELPERS:
        def get_chainDiags(self) -> int:
            """
            Returns the controller state diagnostics. Bit 0 indicates a chain length
            error, bit 1 indicates an inactivity timeout and bit 2 indicates
            a loopback test failure.

            @return an integer corresponding to the controller state diagnostics

            On failure, throws an exception or returns YInputChain.CHAINDIAGS_INVALID.
            """
            return self._run(self._aio.get_chainDiags())

    @classmethod
    def FindInputChain(cls, func: str) -> YInputChain:
        """
        Retrieves a digital input chain for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the digital input chain is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YInputChain.isOnline() to test if the digital input chain is
        indeed online at a given time. In case of ambiguity when looking for
        a digital input chain by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the digital input chain, for instance
                MyDevice.inputChain.

        @return a YInputChain object allowing you to drive the digital input chain.
        """
        return cls._proxy(cls, YInputChain_aio.FindInputChain(func))

    @classmethod
    def FindInputChainInContext(cls, yctx: YAPIContext, func: str) -> YInputChain:
        """
        Retrieves a digital input chain for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the digital input chain is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YInputChain.isOnline() to test if the digital input chain is
        indeed online at a given time. In case of ambiguity when looking for
        a digital input chain by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the digital input chain, for instance
                MyDevice.inputChain.

        @return a YInputChain object allowing you to drive the digital input chain.
        """
        return cls._proxy(cls, YInputChain_aio.FindInputChainInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YInputChainValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _DYNAMIC_HELPERS:
        def resetWatchdog(self) -> int:
            """
            Resets the application watchdog countdown.
            If you have set up a non-zero watchdogPeriod, you should
            call this function on a regular basis to prevent the application
            inactivity error to be triggered.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.resetWatchdog())

    if not _DYNAMIC_HELPERS:
        def get_lastEvents(self) -> str:
            """
            Returns a string with last events observed on the digital input chain.
            This method return only events that are still buffered in the device memory.

            @return a string with last events observed (one per line).

            On failure, throws an exception or returns  YAPI.INVALID_STRING.
            """
            return self._run(self._aio.get_lastEvents())

    def registerStateChangeCallback(self, callback: YStateChangeCallback) -> int:
        """
        Registers a callback function to be called each time that an event is detected on the
        input chain.The callback is invoked only during the execution of
        ySleep or yHandleEvents. This provides control over the time when
        the callback is triggered. For good responsiveness, remember to call one of these
        two functions periodically. To unregister a callback, pass a None pointer as argument.

        @param callback : the callback function to call, or a None pointer.
                The callback function should take four arguments:
                the YInputChain object that emitted the event, the
                UTC timestamp of the event, a character string describing
                the type of event and a character string with the event data.
                On failure, throws an exception or returns a negative error code.
        """
        return self._run(self._aio.registerStateChangeCallback(self._proxyCb(type(self), callback)))

    # --- (end of YInputChain implementation)

